#!/usr/bin/perl
# rjamet, around july 2012
# Script to launch a campaign of simulations in Sinalgo
#
# First option : number of processes to run at the same time

use Parallel::ForkManager; 
#or die("You need Parallel::ForkManager for this, sorry. Try : \ncpan install Parallel::ForkManager");

my $nb_proc = shift; 
my $pm = new Parallel::ForkManager($nb_proc);
my $PWD = join("",`pwd`); chomp $PWD;

sub gen_nodes {
	my $r = "";
	while(@_) {
		my $nb = shift;
		my $cmd = shift;
		$r .= "-gen $nb $cmd " unless ($nb <= 0);
	}
	return " \"$r\" ";
}



$GRAPH_DIR="$PWD/graphs";
$RUNS="-r 20";

# The project Sinalgo will use the config.xml from
$PROJECT=" -p TLCNS ";

# Number of messages to run
$NB_MSG="Messages/Number=".500_000;


#TLCNS settings
$ACK_SIZE = 5; $RET_SIZE = 2; $ROUTE_SIZE = 10; 

$TLCNS_OPT = 
" counter/rolling=true counter/rollingsize=".($ROUTE_SIZE*64). 
" ack/enable=true ack/tl/type=list ack/tl/size=".($ACK_SIZE*64). 
" tl/type=list tl/shared=true tl/size=".($RET_SIZE*64);

#@graphs runs first (graph gen), and waits until everything's done to run @commands
my @graphs;
my @commands;


#The parameters which vary
for my $pctA (5) {
for my $NBNODES (200) {
for my $DEGREE (8) {
for my $FLIPFLOP (100,200,400,800,1600,3200,6400,12800,25600,51200,102400) {
	#number of attackers
	my $nbA = $pctA * $NBNODES / 100;

	#Area of simulation, seed & graph/degree stuff
	my $seed = 0;
	my $AREA_SIDE = int sqrt(($NBNODES*3.14*20*20) / $DEGREE);
	my $LAMBDA = sqrt($DEGREE) / (10 * $NBNODES);
	my $FIELD_SETTINGS=" UDG/rMax=20 dimX=$AREA_SIDE dimY=$AREA_SIDE useFixedSeed=true sender/lambda=$LAMBDA ";

	#shortcuts for node generation
	$GEN_A_TLCNS_SINK="AckingSinkNode Center ";
	$GEN_A_GBR_SINK="GBR:GBRSink Center ";
	$GEN_A_DGBR_SINK="GBR:GBRDupSink Center ";
	$GEN_A_SINK="SinkNode Center ";

	$GEN_RAND_TLCNS="TLCNS:TLCNSNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_GREEDY="Greedy:GreedyNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_RW="RW:RWNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_GBR="GBR:GBRNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_DGBR="GBR:GBRDupNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_CNBACKWARD="CNBackward:CNBackwardNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_BH="BlackholeNode PositionFile \\($GRAPH_DIR/revudg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_DBH="GBR:GBRDupBHNode PositionFile \\($GRAPH_DIR/revudg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_SF="SelectiveForwardingNode PositionFile \\($GRAPH_DIR/revudg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_FBR="FBR:FBRNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_GFG="FBR:GFGNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_SIGF="SIGF:SIGFNode PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_SIGF_BH="SIGF:SIGFBHNode PositionFile \\($GRAPH_DIR/revudg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";

	$GEN_RAND_SIGF_WH="SIGF:SIGFWHNode PositionFile \\($GRAPH_DIR/revudgminusten_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_TLCNS_WH="AckingSinkToBlackNode PositionFile \\($GRAPH_DIR/revudgminusten_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	$GEN_RAND_TLCNS_WH_SWITCH="AckingSinkToFromBlackNode PositionFile \\($GRAPH_DIR/revudgminusten_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";


	#If needed, add the graph generation jobs to @graphs
       if (! -e "./graphs/udg_${NBNODES}_20_${AREA_SIDE}/") {
               unshift @commands, 
                       "cd graphs && ./campaignmaker.pl $NBNODES 20 $AREA_SIDE";
               mkdir "./graphs/udg_${NBNODES}_20_${AREA_SIDE}/";
       }


	#Sinalgo command generation

	push @commands, 
		"perl runner.pl $PROJECT -n \"WHSWITCH-${FLIPFLOP}-SR3_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_TLCNS_SINK,$NBNODES-2*$nbA,$GEN_RAND_TLCNS,$nbA,$GEN_RAND_BH, $nbA,$GEN_RAND_TLCNS_WH_SWITCH).
		"-a \" $NB_MSG $FIELD_SETTINGS $TLCNS_OPT fixedSeed=$seed ack/killackproba=".(1/$NBNODES)." wormhole/flipfloppackets=$FLIPFLOP \" ";

next;
	push @commands, 
		"perl runner.pl $PROJECT -n \"RGBR_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_GBR_SINK,$NBNODES-$nbA,$GEN_RAND_GBR,$nbA,$GEN_RAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed gbr/randomize=true gbr/usetwolevels=false\"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"PRDGBR_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_DGBR_SINK,$NBNODES-$nbA,$GEN_RAND_DGBR,$nbA,$GEN_RAND_DBH).
		"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed gbr/randomize=true gbr/usetwolevels=true\"";

	
	push @commands, 
		"perl runner.pl $PROJECT -n \"GFG_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_SINK,$NBNODES-$nbA,$GEN_RAND_GFG,$nbA,$GEN_RAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS fixedSeed=$seed \"";

	

	push @commands, 
		"perl runner.pl $PROJECT -n \"RW_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-$nbA, $GEN_RAND_RW, $nbA, $GEN_RAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS fixedSeed=$seed\"";


next;
	push @commands, 
		"perl runner.pl $PROJECT -n \"WH-SIGF_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-2*$nbA, $GEN_RAND_SIGF, $nbA, $GEN_RAND_SIGF_BH, $nbA, $GEN_RAND_SIGF_WH).
		"-a \" $NB_MSG $FIELD_SETTINGS fixedSeed=$seed\"";

#push @commands, 
#	"perl runner.pl $PROJECT -n \"FBR_${NBNODES}_${DEGREE}_".$nbA."pct_s$seed\" $RUNS ".
#	"-N ".gen_nodes(1,$GEN_A_SINK,$NBNODES-$nbA,$GEN_RAND_FBR,$nbA,$GEN_RAND_BH).
#	"-a \" $NB_MSG $FIELD_SETTING fixedSeed=$seed \"";



#push @commands, 
#	"perl runner.pl $PROJECT -n \"CNBackward_${NBNODES}_${DEGREE}_".$nbA."pct_s$seed\" $RUNS ".
#	"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-$nbA , $GEN_RAND_CNBACKWARD, $nbA, $GEN_RAND_BH).
#	"-a \" $NB_MSG $FIELD_SETTINGS $CLASSIC  fixedSeed=$seed\"";


#push @commands, 
#	"perl runner.pl $PROJECT -n \"Greedy_${NBNODES}_${DEGREE}_".$nbA."pct_s$seed\" $RUNS ".
#	"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-$nbA, $GEN_RAND_GREEDY, $nbA, $GEN_RAND_BH).
#	"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed Ttl=25\"";


}
}
}
}




#### ACTUAL EXECUTION OF STUFF
for (@graphs)  { #Graph generation first
	if($pm->start) {
			next;

	} else {
			system $_."\n";
			sleep(1);
	}
	$pm->finish;
}

$pm->wait_all_children();#barrier



for (@commands) { #simulations next

	if($pm->start) {
			next;

	} else {

		
		print $_."\n";
		system $_."\n";
		
			sleep(1);
	}
	$pm->finish;
}
$pm->wait_all_children();

`echo "Hello from the imps in \$HOSTNAME. Sinalgo campaign is over." | mailx -s "Sinalgo finish notification" jamet.raphael\@gmail.com`;

#End !
