#!/usr/bin/perl
# rjamet, around july 2012
# Script to launch a campaign of simulations in Sinalgo
#
# First option : number of processes to run at the same time


use Runner::Utils qw/run_many/;
use Runner::Config qw/gen_nodes init_nodes/;
use Cwd; #for getcwd

$GRAPH_DIR=getcwd."/graphs";

#How many runs
$RUNS="-r 10";

# The project Sinalgo will use the config.xml from
$PROJECT=" -p TLCNS ";

# Number of messages to run
my $NB_MSG=" Messages/Number=100000 ";

#TLCNS settings

#@graphs runs first (graph gen), and waits until everything's done to run @commands
my @graphs;
my @commands;


#The parameters which vary
for my $pctA (0) {
	for my $NBNODES (100,200,400) {
		for my $DEGREE (8,16,32) {
			for my $RET_SIZE (1..5) { 

	#number of attackers
	my $nbA = $pctA * $NBNODES / 100;

my $TLCNS_OPT = 
	" counter/rolling=true counter/rollingsize=".(10*64). 
	" ack/enable=true ack/tl/type=perfect ack/tl/size=".($ACK_SIZE*64). 
	" tl/type=list tl/shared=true tl/size=".($RET_SIZE*64).
	" ack/killackproba=0 ";


	#Area of simulation, seed & graph/degree stuff
	my $seed = " useFixedSeed=true fixedSeed=0 ";
	my $AREA_SIDE = int sqrt(($NBNODES*3.14*20*20) / $DEGREE);
	my $LAMBDA = sqrt($DEGREE) / (10 * $NBNODES);
	my $SETTINGS=" UDG/rMax=20 dimX=$AREA_SIDE dimY=$AREA_SIDE $seed sender/lambda=$LAMBDA $NB_MSG";

	init_nodes("$GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/");

	#If needed, add the graph generation jobs to @graphs
       if (! -e "./graphs/udg_${NBNODES}_20_${AREA_SIDE}/") {
               unshift @graphs, 
                       "cd graphs && ./campaignmaker.pl $NBNODES 20 $AREA_SIDE";
               mkdir "./graphs/udg_${NBNODES}_20_${AREA_SIDE}/";
       }


	#Sinalgo command generation

	 push @commands, 
	 	"perl runner.pl $PROJECT -n \"SENT_${RET_SIZE}_SR3_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
	 		gen_nodes(
				1, "S_TLCNS", "CENTER",
	 			$NBNODES, "TLCNS", "HONEST",
#	 			$nbA, "A_BH", "ATTACK",
#	 			$nbA, "A_WHSR3", "ATTACKWH"
	 		).
	 	"-a \"$SETTINGS $TLCNS_OPT\" ";


next;
	 push @commands, 
	 	"perl runner.pl $PROJECT -n \"SIGFA_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
	 		gen_nodes(
				1, "SINK", "CENTER",
	 			$NBNODES-$nbA, "SIGF", "HONEST",
	 			$nbA, "A_SIGFBH_SMART", "ATTACK"
	 		).
	 	"-a \"$SETTINGS\" ";

	 push @commands, 
	 	"perl runner.pl $PROJECT -n \"SIGF_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
	 		gen_nodes(
				1, "SINK", "CENTER",
	 			$NBNODES-$nbA, "SIGF", "HONEST",
	 			$nbA, "A_SIGFBH", "ATTACK"
	 		).
	 	"-a \"$SETTINGS\" ";


	push @commands, 
		"perl runner.pl $PROJECT -n \"GFG_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
			gen_nodes(
				1, "SINK", "CENTER",
				$NBNODES-$nbA, "GFG", "HONEST",
				$nbA, "A_BH", "ATTACK"
			).
		"-a \"  $SETTINGS \"";


next;



	push @commands, 
		"perl runner.pl $PROJECT -n \"RGBR_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
			gen_nodes(
				1, "S_GBR", "CENTER",
				$NBNODES-$nbA, "GBR", "HONEST",
				$nbA, "A_BH", "ATTACK"
			).
		"-a \"  $SETTINGS gbr/randomize=true gbr/usetwolevels=false\"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"PRDGBR_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
			gen_nodes(
				1, "S_DGBR", "CENTER",
				$NBNODES-$nbA, "DGBR", "HONEST",
				$nbA, "A_DBH", "ATTACK"
			).
		"-a \"  $SETTINGS gbr/randomize=true gbr/usetwolevels=true\"";

	
	push @commands, 
		"perl runner.pl $PROJECT -n \"RW_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
			gen_nodes(
				1, "SINK", "CENTER",
				$NBNODES-$nbA, "RW", "HONEST",
				$nbA, "A_BH", "ATTACK"
			).
		"-a \"  $SETTINGS \"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"SR3_${NBNODES}_${DEGREE}_".$pctA."pct\" $RUNS ".
			gen_nodes(
				1, "S_TLCNS", "CENTER",
				$NBNODES-$nbA, "TLCNS", "HONEST",
				$nbA, "A_BH", "ATTACK"
			).
		"-a \"  $SETTINGS $TLCNS_OPT \"";

#next;

			}
		}
	}
}



run_many(@graphs);
run_many(@commands);

`echo "Hello from the imps in \$HOSTNAME. Sinalgo campaign is over." | mailx -s "Sinalgo finish notification" jamet.raphael\@gmail.com`;

#End !
