package Runner::Config;
#Manage configuration for campaigns

use strict;
use warnings;
use Carp qw/ confess /;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(init_nodes gen_nodes);



my %nodes;
my %positions;
my $init_done = 0;

sub init_nodes {

	$init_done = 1;

	my $udgdir = shift or confess();
	my $revudgdir = $udgdir; $revudgdir =~ s/udg/revudg/;
	my $revudg10dir = $udgdir; $revudg10dir =~ s/udg/revudgminusten/;
	my $holeyudgdir1 = $udgdir; $holeyudgdir1 =~ s/udg/holeyudg1/;
	my $holeyudgdir2 = $udgdir; $holeyudgdir2 =~ s/udg/holeyudg2/;

	%nodes = (
		TLCNS => "TLCNS:TLCNSNode",
		SR3 => "TLCNS:TLCNSNode",
		Greedy => "Greedy:GreedyNode",
		GBR => "GBR:GBRNode",
		DGBR => "GBR:GBRDupNode",
		RW => "RW:RWNode",
		FBR => "FBR:FBRNode",
		GFG => "FBR:GFGNode",
		SIGF => "SIGF:SIGFNode",

		A_BH => "BlackholeNode",
		A_DBH => "GBR:GBRDupNode",
		A_SF => "SelectiveForwardingNode",
		A_TLCNSSF => "TLCNS:SFThenTLCNSNode",
		A_SIGFBH => "SIGF:SIGFBHNode",
		A_SIGFBH_SMART => "SIGF:SIGFSmartBHNode",

		A_WH => "SinkToBlackNode",
		A_WHSR3 => "AckingSinkToBlackNode",

		S_TLCNS => "AckingSinkNode",
		S_SR3 => "AckingSinkNode",
		S_GBR => "GBR:GBRSink",
		S_DGBR => "GBR:GBRDupSink",
		SINK => "SinkNode",

		#kind of obsolete
		CNBackward => "CNBackward:CNBackwardNode"
	);

	%positions = (
		HONEST => 'PositionFile \\('.$udgdir.'\\$RUN.pos\\)',
		ATTACK => 'PositionFile \\('.$revudgdir.'\\$RUN.pos\\)',
		ATTACKWH => 'PositionFile \\('.$revudg10dir.'\\$RUN.pos\\)',
		A_HOLE1 => 'PositionFile \\('.$holeyudgdir1.'A\\$RUN.pos\\)',
		A_HOLE2 => 'PositionFile \\('.$holeyudgdir2.'A\\$RUN.pos\\)',
		H_HOLE1 => 'PositionFile \\('.$holeyudgdir1.'H\\$RUN.pos\\)',
		H_HOLE2 => 'PositionFile \\('.$holeyudgdir2.'H\\$RUN.pos\\)',
		CENTER => 'Center ',
		RANDOM => 'Random '
	);
}


sub gen_nodes {

	confess("Do the init first") if (!$init_done);

	my $r = "";
	while(@_) {
		
		my $nb = shift; 
		my $n = shift;
		my $p = shift;

		if (!(defined $n) or !(exists $nodes{$n})) {
			confess("Gen nodes : nref not okay");
		}
		
		if (!(defined $p) or !(exists $positions{$p})) { 
			confess("Gen nodes : pos not okay");
		}

		my $name = $nodes{$n};
		my $pos = $positions{$p};

		$r .= "-gen $nb $name $pos " unless ($nb <= 0);
	}

	return " -N \"$r\" ";
}


1;
