package Runner::Stats;

use strict;
use warnings;
use Carp qw/ confess carp /;
use Runner::IO;
use List::Util qw/reduce sum max min any first/;
use List::MoreUtils qw/first_index/;

my $RA_SAMPLES = 500;
#my $RA_SAMPLES = 10;
my $RD_SAMPLES = 500;
#my $RD_SAMPLES = 10;
my $RE_TIMEWINDOW = 100;

#Used to compute all the means and stddev and so on
sub computeOnMessages($$) {

	my $data = shift;
	my $command = shift;

	if (!exists $data->{inc_vars}) {
		#first run, create the data
		$data->{inc_vars}->{last_id_RA} = 0;
		$data->{inc_vars}->{last_id_RA_ack} = 0;
		$data->{inc_vars}->{last_id_RD} = 0;
		$data->{inc_vars}->{last_id_RD_ack} = 0;
	} else {
		#reuse old data
	}
	
	#Runner::IO::tprint "### Extracting the uniques send times\n";

	$data->{average_rolling} = [] unless exists $data->{average_rolling};
	$data->{average_rolling_ack} = [] unless exists $data->{average_rolling_ack};
	$data->{delivery_rolling} = [] unless exists $data->{delivery_rolling};
	$data->{delivery_rolling_ack} = [] unless exists  $data->{delivery_rolling_ack};
	$data->{energy_rolling} = [] unless exists $data->{energy_rolling};

	###################################
	# NUMBER OF HOPS FOR MSG
	###################################

	#Runner::IO::tprint "### Computing averages on send times\n";


	#The following thing is not noticeably faster than a simple grep, 
	#strangely
	my $lid = $data->{inc_vars}->{last_id_RA};
	my $fi = 
		first_index 
			{ $_->{id} > $lid }  
			@{$data->{msg_data}};

	if (!defined($fi) || $fi eq "") {die()}
		my @sorted;
	if($fi >= 0) {
		@sorted = grep { 

								$_->{d} == 1 && 
								$_->{type} eq "basic"
							} @{$data->{msg_data}}[$fi .. @{$data->{msg_data}}-1 ];


	
		while( scalar @sorted >= $RA_SAMPLES) {
			my @cur = splice @sorted, 0, $RA_SAMPLES;
			my $h_r =  ( sum map{$_->{h}} @cur ) / @cur;
			my $o_r =  ( sum map{$_->{o}} @cur ) / @cur;
			my $t_r =  ( sum map{$_->{tb}} @cur ) / @cur;
			my $tph_r =( sum map{ ($_->{te} - $_->{tb})/$_->{h} } @cur )  / @cur;
	
			push @{$data->{average_rolling}}, {"h" => $h_r, "t" => $t_r, "tph" => $tph_r, "o"=> $o_r};
			$data->{inc_vars}->{last_id_RA} = max map{$_->{id}} @cur;
		}	
}	

	###################################
	# DELIVERY RATE FOR MSG
	###################################

	#Runner::IO::tprint "### Computing average delivery rates\n";
	@sorted = grep { 
								$_->{type} eq "basic" &&
								$_->{id} > $data->{inc_vars}->{last_id_RD}
							} @{$data->{msg_data}};
	while (scalar @sorted >= $RD_SAMPLES) {
		my @cur = splice @sorted, 0, $RD_SAMPLES;
		my $eaten_r = ( scalar grep { $_->{d} == 0 } @cur ) / @cur;
		my $ok_r =    ( scalar grep { $_->{d} == 1 } @cur ) / @cur;
		my $ttl_r =   ( scalar grep { $_->{d} == 2 } @cur ) / @cur;
		my $other_r = ( scalar grep { $_->{d} >= 3 } @cur ) / @cur;

		my $t_r =  ( sum  map{$_->{tb}} @cur ) / @cur;
		push @{$data->{delivery_rolling}}, 
			{"eaten" => $eaten_r, "ok" => $ok_r,"ttl"=>$ttl_r, "other"=>$other_r, "t" => $t_r};		
				
		$data->{inc_vars}->{last_id_RD} = max map{$_->{id}} @cur;
	}

	####################################
	# NBHOPS + DELIVERY RATE FOR ACKS
	####################################


	#FIXME : could be faster by 1/3 if we remove this when not used
	if (any {$_->{type} eq "ack"} @{$data->{msg_data}}) {
		
		my @sorted = grep {
   	          ($_->{d} == 1 || $_->{d} == 3) &&
   	          $_->{id} > $data->{inc_vars}->{last_id_RA_ack} &&
   	          $_->{type} eq "ack"
   	        } @{$data->{msg_data}};

 		while( scalar @sorted >= $RA_SAMPLES) {
	 		my @cur = splice @sorted, 0, $RA_SAMPLES;
 	  	my $h_r =  ( sum map{$_->{h}} @cur ) / @cur;
 	  	my $t_r =  ( sum map{$_->{tb}} @cur ) / @cur;
 	  	my $tph_r =( sum map{ ($_->{te} - $_->{tb})/$_->{h} } @cur )  / @cur;

   		push @{$data->{average_rolling_ack}}, {"h" => $h_r, "t" => $t_r, "tph" => $tph_r};
   		$data->{inc_vars}->{last_id_RA_ack} = max map{$_->{id}} @cur;

		}

		#Runner::IO::tprint "### Computing average delivery rates for acks\n";
		@sorted = grep { 
									$_->{id} > $data->{inc_vars}->{last_id_RD_ack} &&
									$_->{type} eq "ack" 
								} @{$data->{msg_data}};
		while (scalar @sorted >= $RD_SAMPLES) {
			my @cur = splice @sorted, 0, $RD_SAMPLES;
			my $eaten_r = ( scalar grep { $_->{d} == 0 } @cur ) / @cur;
			my $ok_r =    ( scalar grep { $_->{d} == 1 } @cur ) / @cur;
			my $ttl_r =   ( scalar grep { $_->{d} == 2 } @cur ) / @cur;
			my $forgot_r =( scalar grep { $_->{d} == 3 } @cur ) / @cur;
			my $other_r = ( scalar grep { $_->{d} >= 4 } @cur ) / @cur;

			my $t_r =  ( sum  map{$_->{tb}} @cur ) / @cur;
			push @{$data->{delivery_rolling_ack}}, {
				"eaten" => $eaten_r, 
				"ok" => $ok_r,
				"ttl"=>$ttl_r, 
				"for"=>$forgot_r,
				"other"=>$other_r,
				"t" => $t_r
			};

			$data->{inc_vars}->{last_id_RD_ack} = max map{$_->{id}} @cur;
		}
	} else {
		$data->{inc_vars}->{last_id_RD_ack} =  max map{$_->{id}} @{$data->{msg_data}};
		$data->{inc_vars}->{last_id_RA_ack} =  max map{$_->{id}} @{$data->{msg_data}};
	}




	my $last_useful_id = min(
		(	
			$data->{inc_vars}->{last_id_RA},
			$data->{inc_vars}->{last_id_RA_ack},
			$data->{inc_vars}->{last_id_RD},
			$data->{inc_vars}->{last_id_RD_ack}
		)
	);

	my @newdata = grep {$_->{id} > $last_useful_id} @{$data->{msg_data}};
	undef @{$data->{msg_data}};
	$data->{msg_data} = \@newdata;

	1;
}




sub computeAtTheEnd($$) {

	my $data = shift;
	my $command = shift;

	####################################
	# STUFF COMPUTED AFTER EVERYTHING ELSE
	####################################

	if (@{$data->{average_rolling}}) {
		$data->{simulation}->{mean_nbh} =
			( sum map {$_->{h}} @{$data->{average_rolling}} ) / @{$data->{average_rolling}};
		$data->{simulation}->{mean_overhead} =
			( sum map {$_->{o}} @{$data->{average_rolling}} ) / @{$data->{average_rolling}};
	} else {
		Runner::IO::tprint("Problem : too few delivered messages to make an average of the nbhops, possibly they all got lost\n");
		$data->{simulation}->{mean_nbh} = -1;	  
	}

	if (@{$data->{delivery_rolling_ack}}) {
		$data->{simulation}->{total_dr_ack_received} =
			( sum map {$_->{ok} + $_->{for}} @{$data->{delivery_rolling_ack}} ) / @{$data->{delivery_rolling_ack}};
		$data->{simulation}->{total_dr_ack_remembered} =
			( sum map {$_->{ok}}             @{$data->{delivery_rolling_ack}} ) / @{$data->{delivery_rolling_ack}};
	}

	#Fairness as we do it
	my @cleandata = #BEURK
		grep {($_->{t} ne "blackhole") && ($_->{t} ne "sink") && ($_->{t} ne "SFnode")} @{$data->{nodes}};  

	unless(@cleandata) {
		carp("No nodes which are not BH ?\n");
		$data->{simulation}->{nothing_delivered}=1;
		return 1;
	} else {

		$data->{simulation}->{nothing_delivered}=0;

		my $sqfairness = ((sum map {$_->{dr}} @cleandata) / @cleandata)**2;
		my $fairnesssq = (sum map {$_->{dr}**2} @cleandata) / @cleandata;
		$data->{simulation}->{fairness} = sqrt($fairnesssq - $sqfairness);
	}

	#Moustache plot DR/node
	@cleandata = 
		sort {$a <=> $b}
		map {$_->{dr}}
		grep {$_->{t} ne "blackhole" && $_->{t} ne "sink" && ($_->{t} ne "SFnode")} @{$data->{nodes}};
	$data->{simulation}->{boxplot}->[0] = $cleandata[int (@cleandata*0.1)];
	$data->{simulation}->{boxplot}->[1] = $cleandata[int (@cleandata*0.25)];
	$data->{simulation}->{boxplot}->[2] = $cleandata[int (@cleandata*0.5)];
	$data->{simulation}->{boxplot}->[3] = $cleandata[int (@cleandata*0.75)];
	$data->{simulation}->{boxplot}->[4] = $cleandata[int (@cleandata*0.9)];

	#Stacks ORANGE-like
	@cleandata = grep {$_->{t} !~ /blackhole/ && $_->{t} !~ /sink/ && ($_->{t} ne "SFnode")} @{$data->{nodes}};  
	$data->{simulation}->{classes}->[0] = grep {$_->{dr} <= 0} @cleandata;
	$data->{simulation}->{classes}->[1] = grep {$_->{dr} > 0.0    && $_->{dr} <= 0.33} @cleandata;
	$data->{simulation}->{classes}->[2] = grep {$_->{dr} > 0.33 && $_->{dr} <= 0.66} @cleandata;
	$data->{simulation}->{classes}->[3] = grep {$_->{dr} > 0.66 && $_->{dr} < 1} @cleandata;
	$data->{simulation}->{classes}->[4] = grep {$_->{dr} >= 1} @cleandata;

	if ( @{$data->{delivery_rolling}} ) {
		$data->{simulation}->{total_dr} =
			( sum map {$_->{ok}} @{$data->{delivery_rolling}} ) / @{$data->{delivery_rolling}};
		$data->{simulation}->{total_ttl} =
			( sum map {$_->{ttl}} @{$data->{delivery_rolling}} ) / @{$data->{delivery_rolling}};

	} else {
		carp("Everything is terrible, no delivered messages ? ...");
	}

	
	return 1;
}



1;
