#!/usr/bin/perl

#Uses a nodes_data.json file to build a representation
#of the network graph.

use Graph::Layout::Aesthetic;
use Graph::Layout::Aesthetic::Topology;
use JSON::XS;
use SVG;

sub connected {
  my $ax = shift;
  my $ay = shift;
  my $bx = shift;
  my $by = shift;
  return (sqrt( ($ay-$by)**2.0 + ($ax-$bx)**2.0 ) < $udgrad);
}

my $g = Graph::Undirected->new;

my $fh = shift; my $nbh = shift;
my $fa = shift; my $nba = shift;

open(my $hh, $fh) or die($!);
my @xh = (); my @yh = ();
while(<$hh>) {
	chomp;
	if (/^(\d+\.\d*) , (\d+\.\d*),/) {
		push @xh, 0+$1;
		push @yh, 0+$2;
	} 
}
close($hh);

open(my $ha, $fa) or die($!);
my @xa = (); my @ya = ();
while(<$ha>) {
	chomp;
	if (/^(\d+\.\d*) , (\d+\.\d*),/) {
		push @xa, 0+$1;
		push @ya, 0+$2;
	} 
}
close($ha);


for my $a (0..@xh) {
	for my $b (0..@xh) {
		if ($a==$b) {next};
		if (connected($x[$a],$y[$a],$x[$b],$y[$b])) {
			$g->add_edge($a,$b);
		}
	}
}



my $z = 700;

my $countersfile;
open ($countersfile, shift) or die($!);


my @data = <$countersfile>;


my $g = JSON::XS->new->relaxed->decode(join "",@data);

my $topology = Graph::Layout::Aesthetic::Topology->from_graph($g);

$topology->finish;

	$aglo = Graph::Layout::Aesthetic->new($topology);
  $aglo->add_force("node_repulsion");
  $aglo->add_force("min_edge_length");
  $aglo->gloss();
	$aglo->normalize();

	my $svg = SVG->new(width=>$z, height=>$z);

	my $edgegroup = $svg->group(id=>"edge",style=>{"stroke-width"=>0.2,stroke=>'#000'});
	my $BHedgegroup = $svg->group(id=>"bhedge",style=>{fill-opacity=>1.0,'stroke-width'=>0.1,stroke=>'#000','stroke-dasharray'=>"5, 5"});
	my $sinkgroup = $svg->group(id=>'sink',style=>{stroke=>'#000',fill=>"red"});
	my $BHgroup = $svg->group(id=>"bh",style=>{stroke=>'black',fill=>"black"});
	my $vertexgroup = $svg->group(id=>"vertex",style=>{stroke=>'#000',fill=>'#88F'});

  my $i = 0;
  my @coord = $aglo->all_coordinates;
	for my $node (@coord) {
		if(($g->[$i]->{t} eq "tabu" || $g->[$i]->{t} eq "basic")) {
	    $vertexgroup->circle(cx=>$node->[0]*$z, cy=>$node->[1]*$z, r=>3, id=>"c$i");
	  } elsif ($g->[$i]->{t} eq "blackhole") {
			$BHgroup->circle(cx=>$node->[0]*$z, cy=>$node->[1]*$z, r=>3, id=>"c$i");
		} else {  
			$sinkgroup->circle(cx=>$node->[0]*$z, cy=>$node->[1]*$z, r=>7, id=>"c$i",
			onmouseover=>"evt.target.setAttribute('fill', '#0FF');",
			onmouseout=>"evt.target.setAttribute('fill', '#F00');");
		}


			for (@{$g->[$i]->{neighbors}} ) {
				if ($g->[$i]->{t} eq "blackhole" || $g->[$_]->{t} eq "blackhole") {
					if ($g->[$i]->{id} > $_) {
						$BHedgegroup->line(x1=>($coord[$i]->[0])*$z, 
										y1=>($coord[$i]->[1])*$z,
										x2=>($coord[$_-1]->[0])*$z, 
										y2=>($coord[$_-1]->[1])*$z,
										id=>"l$_-$i",
										); 
					}			
				} else {
					if ($g->[$i]->{id} > $_) {
						$edgegroup->line(x1=>($coord[$i]->[0])*$z, 
										y1=>($coord[$i]->[1])*$z,
										x2=>($coord[$_-1]->[0])*$z, 
										y2=>($coord[$_-1]->[1])*$z,
										id=>"l$_-$i",
										); 
					}
				}	

			}
		$i++;

		
  }

open(my $blah, ">graph.svg");
print $blah $svg->xmlify;
close $blah;
use strict;
use Graph;
my $udgrad = 20;

