#!/usr/bin/perl
# rjamet, around july 2012
# Script to launch a campaign of simulations in Sinalgo
#
# First option : number of processes to run at the same time

use strict;

use Parallel::ForkManager; 
#or die("You need Parallel::ForkManager for this, sorry. Try : \ncpan install Parallel::ForkManager");
my $nb_proc = shift; 
my $pm = new Parallel::ForkManager($nb_proc);
my $PWD = join("",`pwd`); chomp $PWD;

sub gen_nodes {
	my $r = "";
	while(@_) {
		my $nb = shift;
		my $cmd = shift;
		$r = "-gen $nb $cmd ".$r unless ($nb <= 0);
	}
	return " \"$r\" ";
}



my $GRAPH_DIR="$PWD/graphs";
my $RUNS="-r 20";

# The project Sinalgo will use the config.xml from
my $PROJECT=" -p TLCNS ";

# Number of messages to run
my $NB_MSG="Messages/Number=".25000;


#TLCNS settings
my $ACK_SIZE = 5; my $RET_SIZE = 3; my $ROUTE_SIZE = 10; 

my $TLCNS_OPT = 
" counter/rolling=true counter/rollingsize=".($ROUTE_SIZE*64). 
" ack/enable=true ack/tl/type=list ack/tl/size=".($ACK_SIZE*64). 
" tl/type=list tl/shared=true tl/size=".($RET_SIZE*64);

#@graphs runs first (graph gen), and waits until everything's done to run @commands
my @graphs;
my @commands;


#The parameters which vary
for my $pctA (5) {
for my $NBNODES (200) {
for my $DEGREE (8) {

	#number of attackers
	my $nbA = $pctA * $NBNODES / 100;

	#Area of simulation, seed & graph/degree stuff
	my $seed = 0;
	my $AREA_SIDE = int sqrt(($NBNODES*3.14*20*20) / $DEGREE);
	my $LAMBDA = sqrt($DEGREE) / (10 * $NBNODES);
	my $FIELD_SETTINGS=" UDG/rMax=20 dimX=$AREA_SIDE dimY=$AREA_SIDE useFixedSeed=true sender/lambda=$LAMBDA";

	#shortcuts for node generation
	my $position = " PositionFile \\($GRAPH_DIR/udg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	my $revposition = " PositionFile \\($GRAPH_DIR/revudg_${NBNODES}_20_${AREA_SIDE}/\\\$RUN.pos\\) ";
	my $poswh = " PositionFile \\($GRAPH_DIR/WH/\\\$RUN.posT\\) ";

	my $GEN_A_TLCNS_SINK=	"AckingSinkNode Center ";
	my $GEN_A_GBR_SINK=		"GBR:GBRSink 		Center ";
	my $GEN_A_DGBR_SINK=	"GBR:GBRDupSink Center ";
	my $GEN_A_SINK=				"SinkNode 			Center ";
	
	my $GEN_RAND_TLCNS=		"TLCNS:TLCNSNode 	$position ";
	my $GEN_RAND_TLCNS_SF="TLCNS:SFThenTLCNSNode $revposition ";

	my $GEN_RAND_TLCN=		"TLCN:TLCNNode 	$position ";
	my $GEN_RAND_TLM=		"TLM:TLMNode 	$position ";

	my $GEN_RAND_GREEDY=	"Greedy:GreedyNode $position ";
	
	my $GEN_RAND_RW=			"RW:RWNode $position ";
	my $GEN_RAND_RW_SF=		"RW:SFThenRWNode $revposition ";
	
	my $GEN_RAND_GBR=			"GBR:GBRNode $position ";
	my $GEN_RAND_GBR_SF=	"GBR:SFThenGBRNode $revposition ";
	 
	my $GEN_RAND_DGBR=		"GBR:GBRDupNode $position ";
	my $GEN_RAND_DBH=			"GBR:GBRDupBHNode $revposition ";
	my $GEN_TRAND_DBH=			"GBR:GBRDupBHNode $poswh ";
	my $GEN_RAND_DGBR_SF=	"GBR:SFThenGBRDupNode $revposition ";
	
	my $GEN_RAND_CNBACKWARD="CNBackward:CNBackwardNode $position ";
	
	my $GEN_RAND_BH=			"BlackholeNode $revposition ";
	my $GEN_TRAND_BH=			"BlackholeNode $poswh ";

	my $GEN_RAND_STB=			"SinkToBlackNode $revposition ";
	my $GEN_RAND_TLCNS_STB=	"AckingSinkToBlackNode $revposition ";
	my $GEN_RAND_GBR_STB=	"GBR:GBRSinkToBlackNode $revposition ";
	my $GEN_RAND_DGBR_STB=	"GBR:GBRDupSinkToBlackNode $revposition ";

	my $GEN_RAND_FBR=			"FBR:FBRNode $position ";
	my $GEN_RAND_GFG=			"FBR:GFGNode $position ";
	my $GEN_RAND_GFG_SF=	"FBR:SFThenGFGNode $revposition ";



	#If needed, add the graph generation jobs to @graphs
       if (! -e "./graphs/udg_${NBNODES}_20_${AREA_SIDE}/") {
               unshift @graphs, 
                       "cd graphs && ./campaignmaker.pl $NBNODES 20 $AREA_SIDE";
               mkdir "./graphs/udg_${NBNODES}_20_${AREA_SIDE}/";
       }


	#Sinalgo command generation

	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-TLCNS_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_TLCNS_SINK,$NBNODES-2*$nbA,$GEN_RAND_TLCNS,$nbA,$GEN_RAND_TLCNS_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS $TLCNS_OPT fixedSeed=$seed \"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-TLM1_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-2*$nbA, $GEN_RAND_TLM, $nbA, $GEN_RAND_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS fixedSeed=$seed tl/size=64 tl/type=list tl/shared=false \"";

	push @commands,
		"perl runner.pl $PROJECT -n \"STB-TLCN8_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_SINK,$NBNODES-2*$nbA,$GEN_RAND_TLCN,$nbA,$GEN_RAND_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS tl/type=list tl/size=".(64*8)." fixedSeed=$seed \"";


	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-GBR_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_GBR_SINK,$NBNODES-2*$nbA,$GEN_RAND_GBR,$nbA,$GEN_RAND_GBR_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed \"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-RGBR_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_GBR_SINK,$NBNODES-2*$nbA,$GEN_RAND_GBR,$nbA,$GEN_RAND_GBR_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed gbr/randomize=true \"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-PRGBR_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_GBR_SINK,$NBNODES-2*$nbA,$GEN_RAND_GBR,$nbA,$GEN_RAND_GBR_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed gbr/randomize=true gbr/usetwolevels=true \"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-PRDGBR_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_DGBR_SINK,$NBNODES-2*$nbA,$GEN_RAND_DGBR,$nbA,$GEN_RAND_DGBR_STB, $nbA, $GEN_TRAND_DBH).
		"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed gbr/randomize=true gbr/usetwolevels=true \"";

	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-GFG_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1,$GEN_A_SINK,$NBNODES-2*$nbA,$GEN_RAND_GFG,$nbA,$GEN_RAND_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS fixedSeed=$seed \"";
	
	push @commands, 
		"perl runner.pl $PROJECT -n \"STB-RW_${NBNODES}_${DEGREE}_".$pctA."pct_s$seed\" $RUNS ".
		"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-2*$nbA, $GEN_RAND_RW, $nbA, $GEN_RAND_STB, $nbA, $GEN_TRAND_BH).
		"-a \" $NB_MSG $FIELD_SETTINGS fixedSeed=$seed\"";
	

#next;

#push @commands, 
#	"perl runner.pl $PROJECT -n \"FBR_${NBNODES}_${DEGREE}_".$nbA."pct_s$seed\" $RUNS ".
#	"-N ".gen_nodes(1,$GEN_A_SINK,$NBNODES-$nbA,$GEN_RAND_FBR,$nbA,$GEN_RAND_BH).
#	"-a \" $NB_MSG $FIELD_SETTING fixedSeed=$seed \"";



#push @commands, 
#	"perl runner.pl $PROJECT -n \"CNBackward_${NBNODES}_${DEGREE}_".$nbA."pct_s$seed\" $RUNS ".
#	"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-$nbA , $GEN_RAND_CNBACKWARD, $nbA, $GEN_RAND_BH).
#	"-a \" $NB_MSG $FIELD_SETTINGS $CLASSIC  fixedSeed=$seed\"";


#push @commands, 
#	"perl runner.pl $PROJECT -n \"Greedy_${NBNODES}_${DEGREE}_".$nbA."pct_s$seed\" $RUNS ".
#	"-N ".gen_nodes(1, $GEN_A_SINK, $NBNODES-$nbA, $GEN_RAND_GREEDY, $nbA, $GEN_RAND_BH).
#	"-a \" $NB_MSG $FIELD_SETTINGS  fixedSeed=$seed Ttl=25\"";


}
}
}




#### ACTUAL EXECUTION OF STUFF
for (@graphs)  { #Graph generation first
	if($pm->start) {
			next;

	} else {
			system $_."\n";
			sleep(1);
	}
	$pm->finish;
}

$pm->wait_all_children();#barrier

for (@commands) { #simulations next

	if($pm->start) {
			next;

	} else {
			system $_."\n";
			sleep(1);
	}
	$pm->finish;
}
$pm->wait_all_children();

`echo "Hello from the imps in \$HOSTNAME. Sinalgo campaign is over." | mailx -s "Sinalgo finish notification" jamet.raphael\@gmail.com`;

#End !
