#!/usr/bin/perl
# rjamet, jan 2012
# Runs a sinalgo session, JSONify the output, decode it, 
# compute some small things, spit out results, and graphs it

package Runner;

use warnings;
use strict;
use Getopt::Std;
use JSON::XS;
use Data::Dumper;
use Carp qw/confess/;
use List::Util;
use Runner::IO;
use Runner::Stats;
use Runner::Job;


$| = 1;
$Getopt::Std::STANDARD_HELP_VERSION = 1;

# Parse the cmdline arguments
my %opts; 
getopt('Nnpard',\%opts); #$opts->{a} is the args

if (!defined($opts{N}) || !defined($opts{n}) || !defined($opts{p})) {
	Runner::IO::runner_usage();
	exit(1);
} 


my @destinations;
if (defined($opts{d})) {
	@destinations = split(/ /, $opts{d});
}

my $tbegin = time;
my ($name, $project) = ($opts{n},$opts{p});


my $nbmsg = 100;
my $nbruns = defined($opts{r}) ? $opts{r} : 10;

Runner::IO::rprint "###### Runner launching campaign ".$opts{n}." \n";
Runner::IO::tprint "### Running Sinalgo... \n";

my $sinalgo_command = "cd .. && ./sinalgo".
	" -batch -project ${project} ".$opts{N}." -overwrite ".$opts{a};

#print $sinalgo_command."\n";

mkdir ("../results");

Runner::IO::tprint "### Command : $sinalgo_command\n";

for my $run (0..$nbruns-1) {	###Running simulations... done !

	Runner::IO::tprint "###### Computing for run $run / $nbruns of ".$opts{n}." command ".$sinalgo_command."\n";

	my $run_name = $name."_".$run;

	my $jhandle = Runner::Job::getJobHandle($sinalgo_command, $run);

	Runner::IO::tprint "### Un-JSONify the results\n";
	my $data = {}; 
	my $mode = -1; # -1 for pre-simulation sinalgo crap

	THIS_SIMULATION: while (<$jhandle>) { #Reading from STDIN
		/^.+$/ or next(); #Ensure we read something more than a \n

		if ($mode == -1) {
			if (/Starting the Simulation\./) {
				$mode = 0; #Switch once we find the right flag
			}
		 	next;
		}

		if (/^===BEGIN MESSAGE DATA===$/) {
			my $json = JSON::XS->new->relaxed->shrink;
			$data->{msg_data} = []; 
			my @todecode = ();

			while(<$jhandle>) {
				if (/^===END MESSAGE DATA===$/) {
					$mode = 1;
					last;
				}

				chomp;
				push (@todecode, $_);
				if(@todecode > 1000) { #wait for a decent amount of messages,
				
					# and then append them to the data already available
					my $rv = eval(' @{$data->{msg_data}} = (@{$data->{msg_data}} , $json->incr_parse(join " ",@todecode)); ');
					if (!defined($rv)) {
						warn "/!\\ Error with the json parsing =======================\n";
						print $@;
						confess;
					}
				
					#clean the buffer
					@todecode = (); #We used the data, reset the array
				
					# then parse them (which will remove those parsed from $data)
					Runner::Stats::computeOnMessages($data, $sinalgo_command);
				}
			}
	
			@{$data->{msg_data}} = (@{$data->{msg_data}} , $json->incr_parse(join " ",@todecode));
			undef @todecode;
			Runner::Stats::computeOnMessages($data, $sinalgo_command);
						#print "Trigger data :".total_size($data).", data->msgdata :".total_size($data->{msg_data}).", data->dr:".total_size($data->{delivery_rolling})."\n";
	
		}	elsif ($mode == 1) {
			my @array = ($_,<$jhandle>);
			my $res = JSON::XS->new->relaxed->decode(join " ",@array);
			for (keys %{$res}) {
				$data->{$_} = $res->{$_};
			}

			last THIS_SIMULATION;
		} else {
			confess("should not happen, strange file format...");
		}
	}

	close($jhandle) or confess("Bad filehandle : ".$!);

	Runner::Stats::computeOnMessages($data, $sinalgo_command);


#	Runner::IO::tprint "### All messages parsed, doing global stuff\n";
	Runner::Stats::computeAtTheEnd($data, $sinalgo_command);



	# Dumping results;

	if($run_name =~ /^\s*$/) {
		die("run name empty ?!");
	}

	`rm -rf ../results/$run_name/`;
	mkdir("../results/$run_name");
	
	Runner::IO::dump_averages($data,$run_name);
	Runner::IO::dump_globals($data,$run_name);
	Runner::IO::dump_node_data($data,$run_name);

	#This would be to dump all the message data, which is huge
	#Runner::IO::dump_raw($data,$run_name);

	undef $data;


	Runner::IO::tprint "### Done with this simulation, onto the next\n";

} #End foreach run of simulation


exit(0);

