package projects.CASTOR.behaviour;

import projects.CASTOR.messages.CastorMessage;
import projects.CASTOR.nodes.nodeImplementations.CastorNode;
import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.messages.BasicMessage.Status;
import projects.common.nodes.BasicNode;
import sinalgo.nodes.edges.Edge;
import sinalgo.tools.statistics.UniformDistribution;

public class CastorMessageBehaviour implements Behaviour {

	CastorNode puppet;

	
	
	@Override
	public void init(BasicNode puppet) {
		assert(puppet instanceof CastorNode):"Should only be used with CastorNodes";
		this.puppet = (CastorNode) puppet;
	}
	
	public boolean shouldIBroadcast(int flowID) {
		double pmax = puppet.getMaxReputation(flowID);
		if (UniformDistribution.nextUniform(0, 1) <
				Math.exp(-puppet.myBroadcastModifier * pmax)) {
			return true;
		} else if (pmax < puppet.myReputationThreshold) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public void act(BasicMessage m) {
		assert(m instanceof CastorMessage):"Bad message type";
		CastorMessage cm = (CastorMessage) m;

		puppet.ackRoutingMap.put(cm.idMessage, cm.lastNode);
		puppet.addTimer(cm);
		
		if(shouldIBroadcast(cm.flowID)) {
//			System.out.println("Castor node "+puppet.ID+" broadcasts for m "+cm.idMessage);
			BasicMessage.duplicatas.put(cm.idMessage, 
					BasicMessage.duplicatas.get(cm.idMessage) + puppet.getConnections().size() - 1 );

			for (Edge e : puppet.getConnections()) {
				CastorMessage cloneCM = (CastorMessage) cm.clone();
				cloneCM.hopto(e);
				puppet.send(cloneCM, e.endNode);
				puppet.ackReputationMap.get(e).add(cm.idMessage);				
			}		
			//Nothing to remember in this case
		} else {
			Edge e = puppet.getBestNode(cm.flowID);
			System.out.println("Castor node "+puppet.ID+" unicasts to "+e.endNode.ID+" for m "+cm.idMessage);
			
			puppet.ackReputationMap.get(e).add(cm.idMessage);
			cm.hopto(e);
			puppet.send(cm, e.endNode);
			//TODO: expiration
		}
		
		
		
	}
	
	
	
	

}
