package projects.FBR.behaviour;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import projects.FBR.messages.FBRMessage;
import projects.common.utils.Tools;

import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import sinalgo.nodes.Position;
import sinalgo.nodes.edges.Edge;

public class FBRRouter implements Behaviour {

	public class SortableEdge implements Comparable<SortableEdge> {

		public Edge e;
		public Position destination;
		public Position origin;

		public SortableEdge(Edge e) {
			this.e = e;
			origin = e.startNode.getPosition();
			destination = e.endNode.getPosition();
		}

		public SortableEdge(Position destination, Position origin) {
			this.destination = destination;
			this.origin = origin;
			for (Edge e : puppet.getConnections()) {
				if (origin == puppet.getPosition() &&
				    destination == e.endNode.getPosition()) {
					this.e = e;
					return;
				}
			}
			return;
		}

		@Override
		public int compareTo(SortableEdge other) {	
			/*
			if (Tools.angle(this.p, other.p, this.o, other.o) == 0 || 
					Tools.angle(this.p, other.p, this.o, other.o) == Math.PI ||
					Tools.angle(this.p, other.p, this.o, other.o) == -Math.PI 
					) {
				return 0;
			} else if (Tools.angle(this.p, other.p, this.o, other.o) < 0 ) {
				return -1;
			} else {
				return 1; 
			}*/
			
			int r = Double.compare(
					Tools.angle(this.destination, new Position(0,1,0), this.origin, new Position(0,0,0)),
					Tools.angle(other.destination, new Position(0,1,0), other.origin, new Position(0,0,0)));
			if (r == 0) {
				return -Double.compare(this.destination.distanceTo(this.origin), 
						other.destination.distanceTo(other.origin));
			}
			return r;
		}

}

	BasicNode puppet;

	@Override
	public void init(BasicNode n) {
		this.puppet = n;
	}

	public SortableEdge CCWnext(ArrayList<SortableEdge> c, SortableEdge f) {

		int pos = Collections.binarySearch(c, f);
		if (pos >= 0) {
			return c.get((pos+1) % c.size());
		} else {
			return c.get((-pos-1) % c.size());
		}
	}

	@Override
	public void act(BasicMessage msg) {
		assert (msg instanceof FBRMessage) : "Bad message type";
		FBRMessage m = (FBRMessage) msg;

		Iterator<Edge> it = puppet.getConnections().iterator();
		Edge nextEdge = null;
		SortableEdge fromEdge;
		if (m.lastNode!=puppet) {
			fromEdge = new SortableEdge(m.lastNode.getPosition(),
				puppet.getPosition());
		} else {
			fromEdge = new SortableEdge(BasicNode.sinkPos,
					puppet.getPosition());
		}
			
		// Generate the CCW connections
		ArrayList<SortableEdge> sortedConnections = new ArrayList<FBRRouter.SortableEdge>();
		if (it.hasNext()) {
			while (it.hasNext()) {
				sortedConnections.add(new SortableEdge(it.next()));
			}
		} else {
			throw new RuntimeException("No neighbors for " + puppet.toString()
					+ " ?");
		}

		Collections.sort(sortedConnections);
		SortableEdge se = CCWnext(sortedConnections, fromEdge);
		nextEdge = se.e;
		
		if (Tools.isIntersecting(nextEdge.startNode.getPosition(), nextEdge.endNode.getPosition(),  m.sender.getPosition(),
				BasicNode.sinkPos)) {
			Position inter = Tools.Intersection(nextEdge.startNode.getPosition(),
					nextEdge.endNode.getPosition(), m.sender.getPosition(),
					BasicNode.sinkPos);
		
			double d = inter.distanceTo(BasicNode.sinkPos);
			if (m.FBRReference == puppet) {
				se = CCWnext(sortedConnections, se);
				nextEdge = se.e;
				m.FBRReference = null;
			}
			if (m.bestIntersectionDistance > d ) {
				m.FBRReference = puppet;
				m.bestIntersectionDistance = d;

			}
		}

		m.hopto(nextEdge);
		puppet.send(m, nextEdge.endNode);
	}
}