package projects.GBR.behaviour;

import java.util.Map;
import java.util.Vector;

import projects.GBR.messages.GBRMessage;
import projects.GBR.nodes.nodeImplementations.GBRDupNode;
import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.messages.BasicMessage.Status;
import projects.common.nodes.BasicNode;
import sinalgo.runtime.Global;

public class GBRTwoLevelProbaDupRouter implements Behaviour {

	GBRDupNode puppet;
	Integer min_distance;
	Double p_SameLevel = 0.4;
	Map<BasicNode, Integer> map;

	@Override
	public void init(BasicNode puppet) {
		assert (puppet instanceof GBRDupNode) : "Puppet needs to be a GBRDupNode";
		this.puppet = (GBRDupNode) puppet;
	}

	public GBRTwoLevelProbaDupRouter(Map<BasicNode, Integer> distances,
			Integer min_distance, Double pSameLevel) {
		this.map = distances;
		this.min_distance = min_distance;
		this.p_SameLevel = pSameLevel;
	}

	@Override
	public void act(BasicMessage m) {
		Vector<BasicNode> vn = new Vector<BasicNode>();
		assert (!(m instanceof GBRMessage));

		BasicMessage m2 = (BasicMessage) m.clone();
		/*
		 * GBR stuff
		 */
		if (min_distance == Integer.MAX_VALUE) {
			for (BasicNode n : map.keySet()) {// dumb GBR
				min_distance = Math.min(map.get(n), min_distance);
			}
			assert (min_distance != Integer.MAX_VALUE) : "GBR routes aren't initialized in node "
					+ puppet.ID + " at time " + Global.currentTime;
		}

		/* Should we bother forwarding : set to false if you 
		 * don't want the optimization of messages already received
		 * (for instance, counting the total number of hops) */
		if (GBRDupNode.sreceived.contains(m.idMessage)) {

			puppet.eatMessage(m, Status.DONOTLOG);
			return;

		} else if (puppet.received.contains(m.idMessage)) {
			int c = GBRDupNode.count.get(m.idMessage);
			if (c == 1) {
				GBRDupNode.count.put(m.idMessage, 0);
				GBRDupNode.sreceived.add(m.idMessage);
				puppet.eatMessage(m, Status.INTERCEPTED);
			} else if (c < 1) {
				throw new RuntimeException("Message wasn't in counter : the count is off");
			} else {
				GBRDupNode.count.put(m.idMessage, c - 1);
				puppet.eatMessage(m, Status.DONOTLOG);
			}
			return;
		}


		puppet.received.add(m.idMessage);

		if (!GBRDupNode.count.containsKey(m.idMessage)) {
			GBRDupNode.count.put(m.idMessage, 1);
		}

		
		//Draw 1
		if (sinalgo.tools.statistics.UniformDistribution.nextUniform(0, 1) < p_SameLevel) {
			for (BasicNode n : map.keySet()) {// dumb GBR
				if (map.get(n) == min_distance + 1) {
					vn.add(n);
				}
			}
		}
		if (vn.isEmpty()) {
			for (BasicNode n : map.keySet()) {// dumb GBR
				if (map.get(n) == min_distance) {
					vn.add(n);
				}
			}
		}

		assert (!vn.contains(puppet));
		assert (!vn.isEmpty());

		int r = (int) sinalgo.tools.statistics.UniformDistribution.nextUniform(
				0, vn.size());
		m.hopto(puppet, vn.get(r));
		puppet.send(m, vn.get(r));

		GBRDupNode.count
				.put(m.idMessage, GBRDupNode.count.get(m.idMessage) + 1);
		
		//BasicNode.rolling_messages++;
		// TODO PAS NET

		//Draw 2
		vn.clear();
		if (sinalgo.tools.statistics.UniformDistribution.nextUniform(0, 1) < p_SameLevel) {
			for (BasicNode n : map.keySet()) {// dumb GBR
				if (map.get(n) == min_distance + 1) {
					vn.add(n);
				}
			}
		}
		if (vn.isEmpty()) {
			for (BasicNode n : map.keySet()) {// dumb GBR
				if (map.get(n) == min_distance) {
					vn.add(n);
				}
			}
		}

		
		r = (int) sinalgo.tools.statistics.UniformDistribution.nextUniform(0,
				vn.size());
		m2.hopto(puppet, vn.get(r));
		puppet.send(m2, vn.get(r));
		return;
	}
}
