package projects.GBR.behaviour;

import java.util.Map;
import java.util.Vector;

import projects.GBR.messages.GBRMessage;
import projects.GBR.nodes.nodeImplementations.GBRNode;
import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import sinalgo.runtime.Global;

public class GBRTwoLevelProbaRouter implements Behaviour {

	GBRNode puppet;
	Double p_upperlevel = 0.5;
	Map<BasicNode, Integer> map;

	@Override
	public void init(BasicNode puppet) {
		assert(puppet instanceof GBRNode);
		this.puppet = (GBRNode) puppet;
	}

	public GBRTwoLevelProbaRouter(Map<BasicNode, Integer> distances, Double pUpper) {
		this.map = distances;
		this.p_upperlevel = pUpper;
	}

	@Override
	public void act(BasicMessage m) {
		Vector<BasicNode> vn = new Vector<BasicNode>();
		assert (!(m instanceof GBRMessage));
		if(puppet.getMinDist()==Integer.MAX_VALUE) {
			for (BasicNode n : map.keySet()) {//dumb GBR
				puppet.setMinDist(Math.min(map.get(n),puppet.getMinDist()));
			}
			assert (puppet.getMinDist()!=Integer.MAX_VALUE)
				:"GBR routes aren't initialized in node "+puppet.ID+" at time "+Global.currentTime;
		}

		int r = (sinalgo.tools.statistics.UniformDistribution.nextUniform(0, 1) > .4) ? 0 : 1;

		for (BasicNode n : map.keySet()) {//RGBR
			if(map.get(n) == puppet.getMinDist() + r) {
				vn.add(n);
			}
		}
		if (vn.isEmpty() && r != 0) {
			for (BasicNode n : map.keySet()) {//dumb GBR
				if(map.get(n) == puppet.getMinDist()) {
					vn.add(n);
				}
			}
		}
		assert (!vn.isEmpty());
		r = (int)sinalgo.tools.statistics.UniformDistribution.nextUniform(0, vn.size());
		m.hopto(puppet,vn.get(r));
		puppet.send(m, vn.get(r));
		return;
	}

}
