package projects.GBR.nodes.nodeImplementations;

import java.awt.Color;

import projects.GBR.behaviour.DedupBlackholeBehaviour;
import projects.GBR.behaviour.DedupReceptionBehaviour;
import projects.GBR.messages.GBRMessage;
import projects.common.RoutingBehaviours.RoutingBehaviour;
import projects.common.behaviours.BroadcastRouter;
import projects.common.behaviours.SilentDropBehaviour;
import projects.common.nodes.BasicNode;
import projects.common.selector.AlwaysSelector;
import projects.common.selector.AndSelector;
import projects.common.selector.BeforeMidSimulationSelector;
import projects.common.selector.DestinationSelector;
import projects.common.selector.FirstHopSelector;
import projects.common.selector.OrSelector;
import projects.common.selector.SecondHopSelector;
import projects.common.selector.TypeSelector;


/**
 * GBRDup kindawormhole : behaves like a sink, until the middle of the 
 * simulation, then becomes a GBRDupBH.
 * @author rjamet
 *
 */

//TODO : uniformize wiht the other STB
public class GBRDupSinkToBlackNode extends GBRDupNode {

	@Override
	public boolean isSink() {
		return new BeforeMidSimulationSelector().activate(null);
	}
	
	@Override
	public boolean sendsMessages() {return false;};

	@Override
	public void init() {
		super.init();

		rb = new RoutingBehaviour(this)
				.add(new AndSelector(new AndSelector(
						new BeforeMidSimulationSelector(), new TypeSelector(
								GBRMessage.class)), 
								new OrSelector(
										new SecondHopSelector(),
										new FirstHopSelector()
										)),
						new BroadcastRouter())
				.add(new AndSelector(new BeforeMidSimulationSelector(),
						new TypeSelector(GBRMessage.class)),
						new SilentDropBehaviour())
				.add(new AndSelector(new BeforeMidSimulationSelector(),
						new DestinationSelector(-1)),
						new DedupReceptionBehaviour())
				.add(new AlwaysSelector(), new DedupBlackholeBehaviour());

	}

	@Override
	protected void startTimers() {

		// initiate startup phase
		
		//Increase rolling messages as sendDirect bypasses the basicnode wrapper for send
		BasicNode.rolling_messages++;
		
		
		//2 for the "wormhole" hop
		this.sendDirect((new GBRMessage(this, this, 2, 0, 0, null)), this);
		super.startTimers();
	}

	@Override
	public String JSONName() {return "sinktoBH";}
	
	@Override
	public Color getColor() {
		return (new BeforeMidSimulationSelector().activate(null)) ? super
				.getColor() : Color.black;
	}
}
