package projects.GBR.nodes.nodeImplementations;

import java.util.HashMap;
import java.util.Map;

import projects.GBR.behaviour.GBRBuilder;
import projects.GBR.behaviour.GBRLowerLevelFirstRouter;
import projects.GBR.behaviour.GBRLowerLevelProbaRouter;
import projects.GBR.behaviour.GBRTwoLevelProbaRouter;
import projects.GBR.messages.GBRMessage;
import projects.common.RoutingBehaviours.RoutingBehaviour;
import projects.common.behaviours.Behaviour;
import projects.common.nodes.BasicNode;
import projects.common.selector.AlwaysSelector;
import projects.common.selector.TypeSelector;
import projects.common.timers.SenderTimer;

/**
 * GradientBasedRoutingNode : exactly what the title said. Will route messages
 * to the first found lower-gradient node. Starts at t=100, so that the
 * gradient-building packets do their jobs before the actual routing starts.
 * 
 * @author rjamet
 * 
 */
public class GBRNode extends BasicNode {
	
	Map<BasicNode, Integer> distanceMap;
	int min_dist = Integer.MAX_VALUE;
	static public boolean bRandomize = true;
	static public boolean bUseTwoLevels = true;
	static public double  pSameLevel = 0.4;

	public int getMinDist() {
		return min_dist;
	}
	public void setMinDist(int md) {
		min_dist = md;
	}
	
	
	@Override
	public void init() {
		super.init();
		distanceMap = new HashMap<BasicNode, Integer>();

		Behaviour b; //The GBR behaviour we want to use
		if (bUseTwoLevels) {
			b = new GBRTwoLevelProbaRouter(distanceMap, pSameLevel);
		} else {
			if (bRandomize) {
				b = new GBRLowerLevelProbaRouter(distanceMap);
			} else {
				b = new GBRLowerLevelFirstRouter(distanceMap);
			}
		}
		
		rb = new RoutingBehaviour(this)
				.add(new TypeSelector(GBRMessage.class), 
					new GBRBuilder(distanceMap))
				.add(
					new AlwaysSelector(),
					b
				);	
		}

	@Override
	protected void startTimers() {
		// source : send messages, but after the GBR builders are supposed done
		(new SenderTimer(this)).startRelative(100, this);
	}
	
	@Override
	public String toString() {
		return super.toString() + "\n mindist : "+ min_dist;
	}
}