package projects.SIGF.behaviour;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;
import java.util.TreeMap;
import java.util.Vector;
import projects.SIGF.nodes.nodeImplementations.SIGFNode;
import projects.SIGF.nodes.nodeImplementations.SIGFNode.RouteTo;
import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import projects.common.utils.*;
import sinalgo.nodes.edges.Edge;

public class SIGFBehaviour implements Behaviour {

	SIGFNode puppet;
	
	RouteTo policy;
	
	TreeMap<Integer, Integer> Nsent = new TreeMap<Integer, Integer>();
	TreeMap<Integer, Integer> Nforward = new TreeMap<Integer, Integer>();
	Integer T = 2;
	
	Queue<Integer> Qsent = new LinkedList<Integer>();

	public SIGFBehaviour(RouteTo who) {
		policy = who;
	}

	@Override
	public void init(BasicNode puppet) {
		this.puppet = (SIGFNode) puppet;
		for (Edge e : puppet.getConnections()) {
			Nsent.put(e.endNode.ID,2);
			Nforward.put(e.endNode.ID,1);
		}
	}
	
	private Edge chooseRandomAccordingToReputation(Collection<Edge> candidates) {
		
		Collection<Edge> candidatesThresholded =new Vector<Edge>();
		Edge bestEdge = null;
		
		for(Edge e : candidates) {
			if (reputation(e) > 0.45) {
				candidatesThresholded.add(e);
			}
			if (reputation(e) > reputation(bestEdge)) {
				bestEdge = e;
			}
		}
		if (!candidatesThresholded.isEmpty()) {
			return puppet.chooseRandomFromCollection(candidatesThresholded);
		} else {
			if (bestEdge != null) {
				return bestEdge;
			} else {
				return puppet.chooseRandomFromCollection(candidates);
			}
		}
		
	}
	
	protected double reputation(Edge e) {
		if (e == null) {return 0;}
		double Nsuccess = (double)Nforward.get(e.endNode.ID) / (double)Nsent.get(e.endNode.ID) ;
		double Nfairness = (T - (double)Nsent.get(e.endNode.ID)) / (double)T;
		
		return (5.0/6.0*Nsuccess + 1.0/6.0 * Nfairness);
	}

	protected Edge route(BasicMessage m ) {

		Collection<Edge> candidates = new Vector<Edge>();
		Iterator<Edge> it;
		
		assert(puppet != null):"Something's seriously wrong";

		if (policy == RouteTo.CONE60D) {/* 1st : 60 deg */
			it = puppet.getConnections().iterator();
			while (it.hasNext()) {
				Edge candidate = it.next();
				double angle = Tools.angle(candidate.endNode.getPosition(),
						BasicNode.sinkPos, puppet.getPosition(),
						puppet.getPosition());
				if (angle <= 1.0/6.0 * Math.PI || angle >= 11.0/6.0 * Math.PI) {
					candidates.add(candidate);
				}
				if (((BasicNode) candidate.endNode).isSink()) {
					return candidate;
				}
			}

		}
		
		if (policy == RouteTo.CLOSER || candidates.isEmpty()) { /* 2nd : closer */
			it = puppet.getConnections().iterator();
			while (it.hasNext()) {
				Edge candidate = it.next();
				if (candidate.endNode.getPosition().distanceTo(
						BasicNode.sinkPos) < puppet.getPosition().distanceTo(
						BasicNode.sinkPos)) {
					candidates.add(candidate);
				}
				if (((BasicNode) candidate.endNode).isSink()) {
					return candidate;
				}
			}
		}

		if (policy == RouteTo.ALL || candidates.isEmpty()) { /* 3rd : the rest */
			it = puppet.getConnections().iterator();
			while (it.hasNext()) {
				Edge candidate = it.next();
				candidates.add(candidate);

				if (((BasicNode) candidate.endNode).isSink()) {
					return candidate;
				}
			}
		}

		return chooseRandomAccordingToReputation(candidates);
	}
	
	
	@Override
	public void act(BasicMessage m) {
		if ((m.hops == 0 || m.destination != null  && m.destination.ID == puppet.ID)) {//for me
			
			Edge next = route(m);
			m.hopto(next);
			m.destination = (BasicNode)next.endNode; //NOT A NO-OP
			
			if (Qsent.size() > 5) {
				Qsent.poll();
			}
			puppet.broadcast(m); 

			Qsent.add(m.idMessage); 
			Nsent.put(m.destination.ID, Nsent.get(m.destination.ID) + 1);
			T++;
			//(double)System.out.println("Reput for "+next.endNode.ID+":"+reputation(next));
	
		} else {//overheard
			
			if(Qsent.contains(m.idMessage)) {
				Qsent.remove((Integer)m.idMessage);
				Nforward.put(m.lastNode.ID, Nforward.get(m.lastNode.ID) + 1);
			}
			puppet.eatMessage(m, projects.common.messages.BasicMessage.Status.DONOTLOG);
			
		}
	}
}
