package projects.TLCNS.behaviour.perNeighborTL;

import projects.TLCNS.messages.AckMessage;
import projects.TLCNS.nodes.nodeImplementations.TLCNSNode;
import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import projects.common.nodes.BasicTabuNode;
import projects.common.utils.TabuList;
import sinalgo.nodes.edges.Edge;
import sinalgo.tools.Tuple;

/**
 * Behaviour of a sink or destination recepting a message
 * 
 * @author rjamet
 * 
 */
public class TLCNSAckRouterBehaviour implements Behaviour {

	BasicTabuNode puppet;
	TabuList tl;

	public TLCNSAckRouterBehaviour(TabuList tl_ack) {
		tl = tl_ack;
	}

	@Override
	public void init(BasicNode n) {
		if (n instanceof BasicTabuNode) {
			this.puppet = (BasicTabuNode) n;
		} else {
			throw new RuntimeException("this needs a BasicTabuNode");
		}
	}

	@Override
	public void act(BasicMessage m) {

		assert (!m.destination.equals(puppet)) : "Message probably not intended for this node";
		assert (m instanceof AckMessage) : "Message isn't a ack";
		AckMessage ack = (AckMessage) m;

		boolean stillNotSent = true;

		for (Edge e : puppet.outgoingConnections) {
			/* Search neighbors for an entry in Mack */
			puppet.initCheck(e);
			Tuple<BasicNode, Integer> t = new Tuple<BasicNode, Integer>(
					(BasicNode) e.endNode, ack.AckID);

			if (tl.isin(t)) {
				tl.remove(t);
				if (stillNotSent) {
					stillNotSent = false;
					if (puppet instanceof TLCNSNode) {
						/* With proba killAckProba, kill the ack */
						if (sinalgo.tools.statistics.UniformDistribution.nextUniform(0, 1) <= TLCNSNode.killAckProba) {
							puppet.eatMessage(ack, BasicMessage.Status.TTL);
						} else {
							ack.hopto(e);
							puppet.send(ack, e.endNode	);
						}
					} else {
						ack.hopto(e);
						puppet.send(ack, e.endNode	);
					}
				}
			}
		}
		if (stillNotSent) { /* Revert to RW, no good entry in Mack */
			if (puppet instanceof TLCNSNode) {
				/* With probability killAckProba, kill the ack */
				if (sinalgo.tools.statistics.UniformDistribution.nextUniform(0, 1) <= TLCNSNode.killAckProba) {
					puppet.eatMessage(ack, BasicMessage.Status.TTL);
				} else {
					Edge e = puppet.chooseRandom(null, puppet.getConnections());
					puppet.send(ack, e.endNode	);
				}
			} else {
				Edge e = puppet.chooseRandom(null, puppet.getConnections());
				puppet.send(ack, e.endNode	);
			}
		}
	}

}
