package projects.TLCNS.behaviour.sharedTL;

import projects.TLCNS.messages.AckMessage;
import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import projects.common.nodes.BasicTabuNode;
import projects.common.utils.TabuList;
import sinalgo.nodes.edges.Edge;
import sinalgo.tools.Tuple;

/**
 * Behaviour of a sink or destination recepting a message
 * 
 * @author rjamet
 * 
 */
public class TLCNSSharedAckReceptionBehaviourRolling implements Behaviour {

	BasicTabuNode puppet;
	TabuList stl;
	TabuList aftl;

	public TLCNSSharedAckReceptionBehaviourRolling(TabuList stl, TabuList rtl) {
		this.stl = stl;
		this.aftl = rtl;
	}

	@Override
	public void init(BasicNode n) {
		if (n instanceof BasicTabuNode) {
			this.puppet = (BasicTabuNode) n;
		} else {
			throw new RuntimeException("CNBackRouter needs a BasicTabuNode");
		}
	}

	@Override
	public void act(BasicMessage m) {

		assert (m.destination.equals(puppet)) : "Message probably not intended for this node";
		assert (m instanceof AckMessage) : "Message isn't a ack";
		AckMessage ack = (AckMessage) m;

		
		for (Edge e : puppet.outgoingConnections) {
			puppet.initCheck(e);
			Tuple<Edge, Integer> t = new Tuple<Edge, Integer>(e, ack.AckID);

			if (stl.isin(t)) {
				aftl.add(e.endNode);
				stl.remove(t);
				puppet.eatMessage(m, BasicMessage.Status.OK);
				return;
			}
		}

		puppet.eatMessage(m, BasicMessage.Status.FORGOT);
	}

}
