package projects.TLCNS.behaviour.sharedTL;

import java.util.Collection;

import projects.common.behaviours.Behaviour;
import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import projects.common.nodes.BasicTabuNode;
import projects.common.utils.TabuList;
import sinalgo.nodes.edges.Edge;
import sinalgo.tools.Tuple;

/**
 * Routes according to TLCNS rules, updates counters/TLs, and stores routes for
 * the acks in another TL
 * 
 * @author rjamet
 * 
 */
public class TLCNSSharedRouterAndAckStorerRolling implements Behaviour {

	BasicTabuNode puppet;
	TabuList tl_expectingack;
	TabuList tl_ackrouting;
	TabuList tl_counter;

	public TLCNSSharedRouterAndAckStorerRolling(TabuList tl_expectingack,
			TabuList tl_ackrouting, TabuList tl_routing) {
		this.tl_expectingack = tl_expectingack; /* ack waiting */
		this.tl_ackrouting = tl_ackrouting; /* ack routing */
		this.tl_counter = tl_routing; /* confidence */
	}

	@Override
	public void init(BasicNode n) {
		if (n instanceof BasicTabuNode) {
			this.puppet = (BasicTabuNode) n;
		} else {
			throw new RuntimeException("CNBackRouter needs a BasicTabuNode");
		}
	}

	@Override
	public void act(BasicMessage m) {

		for (Edge e : puppet.getConnections()) {
			puppet.initCheck(e);
		}

		Edge dest = null;
		double degree = puppet.getConnections().size();

		double sum = 0;
		double bananas = 1.0 / degree;

		Collection<Edge> v = puppet.getConnections();

		if(v.size() == 0) { //Disconnected node
			puppet.eatMessage(m, BasicMessage.Status.TTL);
			return;
		}
		
		for (Edge e : v) {
			sum += tl_counter.count(e.endNode) + bananas;
		}

		double rnd = sinalgo.tools.statistics.UniformDistribution.nextUniform(
				0, sum);
		assert (!Double.isNaN(rnd)) : "NaN generated by RNG : this is bad (infinite counter ?)";

		for (Edge e : v) {
			rnd -= tl_counter.count(e.endNode) + bananas;
			if (rnd <= 0) {
				dest = e;
				break;
			}
		}

		if (m.sender.equals(puppet)) {
			for (Edge e2 : puppet.getConnections()) {
				Tuple<Edge, Integer> tuple = new Tuple<Edge, Integer>(e2,
						m.idMessage);
				tl_expectingack.remove(tuple);
			}

			tl_expectingack.add(new Tuple<Edge, Integer>(dest, m.idMessage));
		}

		/*
		 * update the ack routing TL unless there is already a return path.
		 */
		boolean preexistingInfos = false;
		for (Edge e2 : puppet.getConnections()) {
			if (tl_ackrouting.isin(new Tuple<BasicNode, Integer>(
					(BasicNode) e2.endNode, m.idMessage))) {
				preexistingInfos = true;
			}
		}
		if (!preexistingInfos) {// If this ack was never seen before
			tl_ackrouting.add(new Tuple<BasicNode, Integer>(m.lastNode,
					m.idMessage));
		}
		
		m.hopto(dest);
		puppet.send(m, dest.endNode);

	}

}
