package projects.TLCNS.nodes.nodeImplementations;

import projects.TLCNS.behaviour.perNeighborTL.TLCNSAckReceptionBehaviour;
import projects.TLCNS.behaviour.perNeighborTL.TLCNSAckRouterBehaviour;
import projects.TLCNS.behaviour.perNeighborTL.TLCNSRouter;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedAckReceptionBehaviour;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedAckReceptionBehaviourRolling;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedRouter;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedRouterAndAckStorer;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedRouterAndAckStorerRolling;
import projects.TLCNS.messages.AckMessage;
import projects.common.RoutingBehaviours.RoutingBehaviour;
import projects.common.behaviours.Behaviour;
import projects.common.behaviours.ErrorBehaviour;
import projects.common.behaviours.RWRouter;
import projects.common.behaviours.TTLedBehaviour;
import projects.common.behaviours.TeleportToDestinationRouter;
import projects.common.nodes.BasicTabuNode;
import projects.common.selector.AlwaysSelector;
import projects.common.selector.AndSelector;
import projects.common.selector.DestinationSelector;
import projects.common.selector.TTLSelector;
import projects.common.selector.TypeSelector;
import projects.common.utils.ArrayFifoTabuList;
import projects.common.utils.TabuList;
import sinalgo.nodes.edges.Edge;

/**
 * 
 * TLCNSNode : A node that follows the TLCNS routing protocol.
 * 
 * @author rjamet
 * 
 */
public class TLCNSNode extends BasicTabuNode {

	static public boolean sharedtl = true;
	static public boolean teleportAcks = false;
	public static boolean useAckTL = true;
	public static TabuList.Type ackTlType;
	public static int ackTlSize;
	
	public static double killAckProba = 0;
	
	static public boolean rollingCounters = false;
	public static int rollingCountersSize = 1280;
	public TabuList tl_routing;

	@Override
	public void init() {
		super.init();

		tl_routing = new ArrayFifoTabuList(rollingCountersSize);
		
		Behaviour bAck, bMess, bAckReception;
		if (useAckTL) {
			TabuList tl_ackRouting;
			tl_ackRouting = TabuList.build(ackTlType, ackTlSize);
			bAck = new TLCNSAckRouterBehaviour(tl_ackRouting);
			if (sharedtl) {
				TabuList tl_expectingAck;
				tl_expectingAck = TabuList.build();
				if (rollingCounters) {		
					bAckReception = new TLCNSSharedAckReceptionBehaviourRolling(tl_expectingAck,tl_routing);
					bMess = new TLCNSSharedRouterAndAckStorerRolling(tl_expectingAck, tl_ackRouting, tl_routing);
				} else {
					bAckReception = new TLCNSSharedAckReceptionBehaviour(tl_expectingAck);
					bMess = new TLCNSSharedRouterAndAckStorer(tl_expectingAck, tl_ackRouting);
				}
			} else {//not yet done
				assert(false):"WIP";
				bAckReception = new ErrorBehaviour("This choice of parameters is still WIP");
				bMess = new ErrorBehaviour("This choice of parameters is still WIP");
			}
		} else if (teleportAcks) {
			bAck = new TeleportToDestinationRouter();
			if (sharedtl) {
				TabuList tl_expectingAck;
				tl_expectingAck = TabuList.build();
				bMess = new TLCNSSharedRouter(tl_expectingAck);
				bAckReception = new TLCNSSharedAckReceptionBehaviour(tl_expectingAck);
			} else {
				bAckReception = new TLCNSAckReceptionBehaviour();
				bMess = new TLCNSRouter();
			}
		} else {
			bAck = new RWRouter();
			if (sharedtl) {
				TabuList tl_expectingAck;
				tl_expectingAck = TabuList.build();
				bMess = new TLCNSSharedRouter(tl_expectingAck);
				bAckReception = new TLCNSSharedAckReceptionBehaviour(tl_expectingAck);
			} else {
				bAckReception = new TLCNSAckReceptionBehaviour();
				bMess = new TLCNSRouter();
			}
		}

		rb = new RoutingBehaviour(this)
				.add(new TTLSelector(), new TTLedBehaviour())
				.add(new AndSelector(new TypeSelector(AckMessage.class),
						new DestinationSelector(this.ID)), bAckReception)
				.add(new TypeSelector(AckMessage.class), bAck)
				.add(new AlwaysSelector(), bMess);
		
	}

	@Override
	public void JSONOutput() {

		assert (sent_count >= delivered_count) : "Plus de messages arriv�s que d'envoy�s";

		if (sent_count == 0) {
			sent_count++; // division par 0
		}

		System.out.println("{\"id\":"
				+ ID
				+ ", \"t\":\""+ JSONName() + "\", \"d\":"
				+ this.outgoingConnections.size()
			//	+ ", \"sharedTL\" : \""
			//	+ sharedtl
			//	+ "\","
				+ ", \"dr\":"
				+ projects.common.utils.Tools
						.tojson((delivered_count / (sent_count))) + ","
				/*+ JSONCounters() + ","*/ + JSONNeighbors() + "},");
	}

	@Override
	/**
	 * Initializes the counters and TLs for the edge
	 * Overridden to also set shared TLs
	 */
	public void initCheck(Edge e) { 
		double initval = CounterInit/this.getConnections().size();

		if (sharedtl) {
			if (counters.get(e) == null) {
				counters.put(e, initval);
				}
		} else {
			if (tl.get(e) == null || counters.get(e) == null) {
				tl.put(e, TabuList.build());
				counters.put(e, initval);
			}
		}

	}

}
