package projects.TLCNS.nodes.nodeImplementations;

import java.awt.Color;

import projects.TLCNS.behaviour.perNeighborTL.TLCNSAckReceptionBehaviour;
import projects.TLCNS.behaviour.perNeighborTL.TLCNSAckRouterBehaviour;
import projects.TLCNS.behaviour.perNeighborTL.TLCNSRouter;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedAckReceptionBehaviour;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedAckReceptionBehaviourRolling;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedRouter;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedRouterAndAckStorer;
import projects.TLCNS.behaviour.sharedTL.TLCNSSharedRouterAndAckStorerRolling;
import projects.TLCNS.messages.AckMessage;
import projects.common.RoutingBehaviours.RoutingBehaviour;
import projects.common.behaviours.Behaviour;
import projects.common.behaviours.BlackholeBehaviour;
import projects.common.behaviours.ErrorBehaviour;
import projects.common.behaviours.RWRouter;
import projects.common.behaviours.TeleportToDestinationRouter;
import projects.common.selector.AlwaysSelector;
import projects.common.selector.AndSelector;
import projects.common.selector.BeforeTimeSelector;
import projects.common.selector.DestinationSelector;
import projects.common.selector.TypeSelector;
import projects.common.utils.TabuList;
import sinalgo.runtime.Global;

/**
 * 
 * TLCNSNode : A node that follows the TLCNS routing protocol.
 * 
 * @author rjamet
 * 
 */
public class TLCNSToBlackNode extends TLCNSNode {

	@Override
	public void init() {
		super.init();

		Behaviour bAck, bMess, bAckReception;
		if (useAckTL) {
			TabuList atl;
			atl = TabuList.build(ackTlType, ackTlSize);
			bAck = new TLCNSAckRouterBehaviour(atl);
			if (sharedtl) {
				TabuList stl;
				stl = TabuList.build();
				if (rollingCounters) {		
					bAckReception = new TLCNSSharedAckReceptionBehaviourRolling(stl,tl_routing);
					bMess = new TLCNSSharedRouterAndAckStorerRolling(stl, atl, tl_routing);
				} else {
					bAckReception = new TLCNSSharedAckReceptionBehaviour(stl);
					bMess = new TLCNSSharedRouterAndAckStorer(stl, atl);
				}
			} else {//not yet done
				bAckReception = new ErrorBehaviour("WIP");
				bMess = new ErrorBehaviour("WIP");
			}
		} else if (teleportAcks) {
			bAck = new TeleportToDestinationRouter();
			if (sharedtl) {
				TabuList stl;
				stl = TabuList.build();
				bMess = new TLCNSSharedRouter(stl);
				bAckReception = new TLCNSSharedAckReceptionBehaviour(stl);
			} else {
				bAckReception = new TLCNSAckReceptionBehaviour();
				bMess = new TLCNSRouter();
			}
		} else {
			bAck = new RWRouter();
			if (sharedtl) {
				TabuList stl;
				stl = TabuList.build();
				bMess = new TLCNSSharedRouter(stl);
				bAckReception = new TLCNSSharedAckReceptionBehaviour(stl);
			} else {
				bAckReception = new TLCNSAckReceptionBehaviour();
				bMess = new TLCNSRouter();
			}
		}

		rb = new RoutingBehaviour(this)
				.add(new AndSelector(
						new BeforeTimeSelector(10000),new AndSelector(new TypeSelector(AckMessage.class),
						new DestinationSelector(this.ID))), bAckReception)
				.add(new AndSelector(
						new BeforeTimeSelector(10000),new TypeSelector(AckMessage.class)), bAck)
				.add(new BeforeTimeSelector(10000), bMess)
				.add(new AlwaysSelector(), new BlackholeBehaviour());
	}
	
	@Override
	public Color getColor() {
		if (Global.currentTime < 10000) {
			return super.getColor();
		} else {
			return Color.black;
		}
	}

	@Override
	public boolean sendsMessages() {
		return (Global.currentTime < 10000 && super.sendsMessages());
	}

	@Override
	public String JSONName() {
		return "TLCNStoBlackNode";
	}
	
	
}

