package projects.common.RoutingBehaviours;

import java.util.LinkedList;

import projects.common.behaviours.Behaviour;
import projects.common.behaviours.ErrorBehaviour;
import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import projects.common.selector.Selector;

import sinalgo.tools.Tuple;

/**
 * A class which represents the routing behaviour of a node. Add MessageRouters
 * linked to MessageSelectors to build a complete node specification. They will
 * be evaluated in order.
 * 
 * @author rjamet
 * 
 */
public class RoutingBehaviour {

	LinkedList<Tuple<Selector, Behaviour>> behaviours;
	BasicNode puppet;
	
	Behaviour lastcase = new ErrorBehaviour("Uncatched message");
	
	public RoutingBehaviour(BasicNode puppet) {
		this.puppet = puppet;
		behaviours = new LinkedList<Tuple<Selector,Behaviour>>();
	}
	
	public void init() {
		for(Tuple<Selector, Behaviour> b : behaviours) {
			b.first.init();
			b.second.init(puppet);
		}
		lastcase.init(puppet);
	}

	/**
	 * Adds the given filter/router to this behaviour
	 * @param ms the filter
	 * @param rb the router
	 */
	public RoutingBehaviour add(Selector ms, Behaviour rb) {
		behaviours.add(new Tuple<Selector, Behaviour>(ms, rb));
		return this;
	}

	/**
	 * Returns the destination edge, or null if there is no valid choice
	 * <b>may have collateral effects</b>
	 * 
	 * @param m the message to route
	 * @return the destination edge
	 */
	public void act(BasicMessage m) {
		for (Tuple<Selector, Behaviour> t : behaviours) {
			if (t.first.activate(m)) {
				t.second.act(m);
				return;
			}
		}
		lastcase.act(m);
	}
	


}
