package projects.common.timers;

import projects.common.messages.*;
import projects.common.nodes.*;
import projects.common.utils.Tools;
import sinalgo.configuration.CorruptConfigurationEntryException;
import sinalgo.nodes.messages.Message;
import sinalgo.nodes.timers.Timer;
import sinalgo.runtime.Global;
import sinalgo.tools.statistics.Distribution;
import sinalgo.tools.statistics.ExponentialDistribution;

/**
 * Timer used to send messages
 * 
 * @author rjamet
 */
public class SenderTimer extends Timer {

	/**
	 * The node to which this timer is attached.
	 */
	public BasicNode sender;

	/**
	 * Identifier counter
	 */
	public static int id_message = 1;

	/**
	 * Number of messages to send
	 */
	public static int nb_message = 0;

	/**
	 * Number of maximal messages to send
	 */
	public static int nb_message_max = 100000;

	public static boolean firstglobalrun = true;
	public static Distribution dist;

	public SenderTimer(BasicNode sender) {
		this.sender = sender;
		if (firstglobalrun) {
			firstglobalrun = false;
			try {
				dist = Distribution.getDistributionFromConfigFile("Sender");
			} catch (CorruptConfigurationEntryException e1) {
				dist = new ExponentialDistribution(0.1);
				e1.printStackTrace();
			}

			try {
				if (sinalgo.configuration.Configuration
						.hasParameter("Messages/Number")) {
					nb_message_max = sinalgo.configuration.Configuration
							.getIntegerParameter("Messages/Number");

				}
			} catch (CorruptConfigurationEntryException e) {
				System.err
						.println("config.xml corrupted, reverting to default params");
			}
		}
	}

	@Override
	public void fire() {

		if (nb_message <= nb_message_max) {

			if (sender.sendsMessages()) {
				Message msg = genMessage(null);

				sender.sendDirect(msg, sender);
				sender.sent_count++;
				BasicNode.rolling_messages++;

			}

			this.startRelative(dist.nextSample(), node);
			// recursive restart of the timer
		}

	}

	protected BasicMessage genMessage(BasicNode to) {
		nb_message++;
		return new BasicMessage(sender, sender, 0, Global.currentTime,
				Tools.generateID(), to);

	}

	public static boolean hasSentAllRequired() {
		return nb_message > nb_message_max;
	}
}
