package projects.common.utils;

import projects.GBR.nodes.nodeImplementations.GBRNode;
import projects.RW.nodes.nodeImplementations.SFThenRWNode;
import projects.TLCNS.nodes.nodeImplementations.TLCNSNode;
import projects.common.MessageLogger;
import projects.common.nodes.BasicNode;
import projects.common.nodes.BasicTabuNode;
import projects.defaultProject.nodes.nodeImplementations.AckingSinkToFromBlackNode;
import sinalgo.configuration.CorruptConfigurationEntryException;

/**
 * Static class which is the interface between the config.xml and CLI
 * configuration and the java variables
 * 
 * @author rjamet
 * 
 */
public class ConfigHelper {

	private ConfigHelper() {
		assert (false) : "Shouldn't be instanciated";
	}

	private static boolean getConfig(String n, boolean defaultValue) {

		boolean b = defaultValue;
		try {
			if (sinalgo.configuration.Configuration.hasParameter(n)) {
				b = sinalgo.configuration.Configuration.getBooleanParameter(n);
			}
		} catch (CorruptConfigurationEntryException e) {
			System.err
					.println("config.xml corrupted, reverting to default params");
		}
		return b;
	}

	@SuppressWarnings("unused")
	private static String getConfig(String n, String defaultValue) {

		String b = defaultValue;
		try {
			if (sinalgo.configuration.Configuration.hasParameter(n)) {
				b = sinalgo.configuration.Configuration.getStringParameter(n);
			}
		} catch (CorruptConfigurationEntryException e) {
			System.err
					.println("config.xml corrupted, reverting to default params");
		}
		return b;
	}

	private static Integer getConfig(String n, Integer defaultValue) {

		Integer b = defaultValue;
		try {
			if (sinalgo.configuration.Configuration.hasParameter(n)) {
				b = sinalgo.configuration.Configuration.getIntegerParameter(n);
			}
		} catch (CorruptConfigurationEntryException e) {
			System.err
					.println("config.xml corrupted, reverting to default params");
		}
		return b;
	}

	private static Double getConfig(String n, Double defaultValue) {

		Double b = defaultValue;
		try {
			if (sinalgo.configuration.Configuration.hasParameter(n)) {
				b = sinalgo.configuration.Configuration.getDoubleParameter(n);
			}
		} catch (CorruptConfigurationEntryException e) {
			System.err
					.println("config.xml corrupted, reverting to default params");
		}
		return b;
	}

	private static TabuList.Type getConfig(String n, TabuList.Type defaultValue) {

		TabuList.Type b = defaultValue;
		try {
			if (sinalgo.configuration.Configuration.hasParameter(n)) {
				String t = sinalgo.configuration.Configuration
						.getStringParameter(n);
				if (t == null) {
					b = TabuList.Type.INVALID;
				} else if (t.equals("bloom")) {
					b = TabuList.Type.BLOOM;
				} else if (t.equals("list") || t.equals("liste")
						|| t.equals("lrulist")) {
					b = TabuList.Type.LIST_LRU;
				} else if (t.equals("fifolist")) {
					b = TabuList.Type.LIST_FIFO;
				} else if (t.equals("randlist")) {
					b = TabuList.Type.LIST_RAND;
				} else if (t.equals("perfect") || t.equals("perf")
						|| t.equals("infinite")) {
					b = TabuList.Type.INFINITE;
				} else {
					b = TabuList.Type.INVALID;
				}
			}
		} catch (CorruptConfigurationEntryException e) {
			System.err
					.println("config.xml corrupted, reverting to default params");
		}
		return b;
	}

	/**
	 * parseConfig is the function using all the configuration entries for my
	 * packages
	 */
	public static void parseConfig() {

		/*
		 * FIXME beurk !
		 */

		MessageLogger.init();

		BasicNode.Ttl = ConfigHelper.getConfig("ttl", 1000);
		BasicNode.noBacktrack = ConfigHelper.getConfig("rw/nobacktrack", false);
		BasicNode.useLD = ConfigHelper.getConfig("rw/useld", false);

		BasicTabuNode.alpha = ConfigHelper.getConfig("counter/alpha", 1.0);

		TabuList.type = ConfigHelper
				.getConfig("tl/type", TabuList.Type.INVALID);
		TabuList.size = ConfigHelper.getConfig("tl/size", -1);
		TLCNSNode.sharedtl = ConfigHelper.getConfig("tl/shared",
				TLCNSNode.sharedtl);

		TLCNSNode.useAckTL = ConfigHelper.getConfig("ack/enable",
				TLCNSNode.useAckTL); /* ack routing TL */
		TLCNSNode.teleportAcks = ConfigHelper.getConfig("ack/teleport",
				TLCNSNode.teleportAcks);
		TLCNSNode.ackTlType = ConfigHelper.getConfig("ack/tl/type",
				TabuList.Type.LIST_FIFO);
		TLCNSNode.ackTlSize = ConfigHelper.getConfig("ack/tl/size", -1);

		TLCNSNode.rollingCounters = ConfigHelper.getConfig("counter/rolling",
				TLCNSNode.rollingCounters);
		TLCNSNode.rollingCountersSize = ConfigHelper.getConfig(
				"counter/rollingsize", TLCNSNode.rollingCountersSize);
		TLCNSNode.killAckProba = ConfigHelper.getConfig(
				"ack/killackproba", 0.0);

		GBRNode.bRandomize = ConfigHelper.getConfig("gbr/randomize", false);
		GBRNode.bUseTwoLevels = ConfigHelper.getConfig("gbr/usetwolevels",
				false);
		

		SFThenRWNode.interceptProbability = ConfigHelper.getConfig(
				"selectiveforwarding/probability", 0.5);
		AckingSinkToFromBlackNode.packetAmount = ConfigHelper.getConfig("wormhole/flipfloppackets", 100000);

	}

}
