package projects.common.utils;

public abstract class TabuList {

	public static enum Type {
		INFINITE, BLOOM, LIST_RAND, LIST_FIFO, LIST_LRU, INVALID		
	}

	public static int size = -1;
	public static Type type = Type.INVALID;

	// Adds an element to the TL and updates
	public abstract void add(Object i);

	public abstract void remove(Object i);

	// cleans the tl
	public abstract void empty();

	public static TabuList build() {
		sanity();
		return build(type, size);
	}

	private static void sanity() {
		if (type == Type.INVALID || (type != Type.INFINITE && size < 0)) {
			throw new RuntimeException("TL params mis-initialized (no sink ?)");
		}
	}

	public static TabuList build(Type type, int size) {
		if (type == Type.BLOOM) {
			return new BloomTabuList(size);
		} else if (type == Type.LIST_LRU) {
			return new ArrayLRUTabuList(size);
		} else if (type == Type.LIST_RAND) {
			return new ArrayRandTabuList(size);
		} else if (type == Type.LIST_FIFO) {
			return new ArrayFifoTabuList(size);
		} else if (type == Type.INFINITE) {
			return new PerfectTabuList(0);
		} else {
			assert false : "Unknown tabu list type : " + type;
			return null;
		}
	}

/*public static int parseConfigType() {
		String t = ConfigHelper.getConfig("TL/Type", "invalid");
		if (t == null) {
			type = TYPE_INVALID;
		} else if (t.equals("bloom")) {
			type = TabuList.TYPE_BLOOM;
		} else if (t.equals("list") || t.equals("lrulist")) {
			type = TabuList.TYPE_LIST_LRU;
		} else if (t.equals("fifolist")) {
			type = TabuList.TYPE_LIST_FIFO;
		} else if (t.equals("randlist")) {
			type = TabuList.TYPE_LIST_RAND;
		} else if (t.equals("perfect") || t.equals("perf")) {
			type = TabuList.TYPE_PERFECT;
		} else {
			type = TYPE_INVALID;
		}
		return type;
	}

	public static int parseConfigSize() {
		size = ConfigHelper.getConfig("tl/size", -1);
		return size;
	}*/

	public abstract boolean isin(Object i);

	public abstract int count(Object i);
}
