package projects.common.utils;


import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;

import projects.common.messages.BasicMessage;
import projects.common.nodes.BasicNode;
import projects.common.timers.SenderTimer;
import sinalgo.nodes.Position;
import sinalgo.nodes.edges.Edge;

public class Tools {

	public static int generateID() {//FIXME

		int id = (SenderTimer.id_message)++;
		BasicMessage.duplicatas.put(id,1);
		BasicMessage.one_delivered.put(id,false);
		return id;
		
	}

	/**
	 * rounds a double to something a bit shorter
	 * 
	 */
	public static String tojson(double d) {
		DecimalFormat df = new DecimalFormat();
		df.setMinimumFractionDigits(3);
		df.setDecimalFormatSymbols(new DecimalFormatSymbols(new Locale("en")));
		df.setGroupingUsed(false);
		return df.format(d);

		// le saviez vous ? float.tostring ne prend pas les locales en compte,
		// mais DecimalFormat si !
	}

	public static boolean isIntersecting(Edge e1, Edge e2) {
		return isIntersecting(e1.startNode.getPosition(),
				e1.endNode.getPosition(), e2.startNode.getPosition(),
				e2.endNode.getPosition());
	}

	public static boolean isIntersecting(Edge e1, Position e2a, Position e2b) {
		return isIntersecting(e1.startNode.getPosition(),
				e1.endNode.getPosition(), e2a, e2b);
	}

	
	/**
	 * @param A
	 * @param B first segment
	 * @param C
	 * @param D second segment
	 * @return true iff [AB] intersects [CD]
	 */
	public static boolean isIntersecting(Position A, Position B,
			Position C, Position D) {
		if ( 
				/* C and D on different sides of (AB)*/ 
				(CCW(A, B, C)<0) ^ (CCW(A, B, D) < 0) 
				&&  /* A and B on different sides of (CD)*/ 
				(CCW(C, D, A)<0) ^ (CCW(C, D, B)) < 0) {
			return true;
		} else {
			return false;
		}
	}

	public static Position Intersection(Position e1a, Position e1b,
			Position e2a, Position e2b) {
		double x1 = e1a.xCoord, x2 = e1b.xCoord, x3 = e2a.xCoord, x4 = e2b.xCoord;
		double y1 = e1a.yCoord, y2 = e1b.yCoord, y3 = e2a.yCoord, y4 = e2b.yCoord;

		/* How would we live without Wikipedia */
		double x = ((x1 * y2 - y1 * x2) * (x3 - x4) - (x1 - x2)
				* (x3 * y4 - y3 * x4))
				/ ((x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4));
		double y = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2)
				* (x3 * y4 - y3 * x4))
				/ ((x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4));
		return new Position(x, y, 0);
	}

	/* Positive if the angle ABC is CCW, negative if CW, probably 0 otherwise
	 * Seems to work, not sure of robustness
	 */
	public static double CCW(Position a, Position b, Position c) {
		return (b.xCoord - a.xCoord) * (c.yCoord - a.yCoord)
				- (c.xCoord - a.xCoord) * (b.yCoord - a.yCoord);
	}

	/**
	 * @param A
	 *            1st point
	 * @param B
	 *            2nd point
	 * @param OA
	 *            Origin for A
	 * @param OB
	 *            Origin for B
	 * @return Relative angle in radians between vectors --0A,A-> and --OB,B->.
	 *         That return value is in ]-pi;+pi].
	 *         Seems to work for all my purposes, feel free to correct me
	 */
	public static double angle(Position A, Position B, Position OA, Position OB) {
		double Ax = A.xCoord - OA.xCoord;
		double Ay = A.yCoord - OA.yCoord;

		double Bx = B.xCoord - OB.xCoord;
		double By = B.yCoord - OB.yCoord;

		double res = (Math.atan2(By, Bx) - Math.atan2(Ay, Ax));
		if (res > Math.PI) {
			return res - 2 * Math.PI;
		} else if (res <= -Math.PI) {
			return res + 2 * Math.PI;
		} else {
			return res;
		}
	}

	public static double distancesq(Position a, Position b) {
		return a.distanceTo(b);
	}

	public static Position midpoint(Position a, Position b) {
		return new Position((a.xCoord + b.xCoord) / 2,
				(a.yCoord + b.yCoord) / 2, (a.zCoord + b.zCoord) / 2);
	}

	public static void buildBlacklistPlanarize() {
		NODE1: for (BasicNode n1 : BasicNode.list) {
			if (n1 == null) {
				continue NODE1;
			}
			Collection<Edge> ce = n1.getConnections();
			CONNECTIONS: for (Edge e : ce) {
				BasicNode n2 = (BasicNode) e.endNode;
				double radsq = distancesq(
						midpoint(n1.getPosition(), n2.getPosition()),
						n2.getPosition());
				CANDIDATE: for (BasicNode n3 : BasicNode.list) {
					if (n3 == null) {
						continue CANDIDATE;
					}
					if (distancesq(
							midpoint(n1.getPosition(), n2.getPosition()),
							n3.getPosition()) < radsq - 0.001) {
						n1.blacklist.add(e);
						n2.blacklist.add(e.oppositeEdge);
						continue CONNECTIONS;
					}
				}
			}
		}
	}

	public static void buildBlacklistPlanarize_2() {
		ArrayList<Edge> ae = new ArrayList<Edge>();
		ArrayList<Edge> removed = new ArrayList<Edge>();

		for (BasicNode n : BasicNode.list) {
			if (n != null) {
				for (Edge e : n.getConnections()) {
					if (!ae.contains(e)) {
						ae.add(e);
					}
				}
			}
		}
		// First pass : remove intersections
		boolean ended = false;
		while (!ended) {
			Iterator<Edge> i1 = ae.iterator();
			A: while (i1.hasNext()) {
				Edge e = i1.next();
				for (Edge e2 : ae) {
					if (isIntersecting(e, e2)) {
						// Forbid e and forget it for the planarization
						// Runtime.removeEdge(e);
						((BasicNode) e.endNode).blacklist.add(e.oppositeEdge);
						((BasicNode) e.startNode).blacklist.add(e);
						ae.remove(e);
						ae.remove(e.oppositeEdge);
						removed.add(e);
						removed.add(e.oppositeEdge);
						break A; //restart the whole process until it works
					}

				}
				if (i1.hasNext() == false)
					ended = true;
			}
		}

		// Second pass : re-add all that can be added
		// Maybe shuffle the removed array ?
		Iterator<Edge> i2 = removed.iterator();
		A: while (i2.hasNext()) {
			Edge e = i2.next();
			for (Edge e2 : ae) {
				if (isIntersecting(e, e2)) {
					continue A;
				}
			}
			ae.add(e);
			ae.add(e.oppositeEdge);
			((BasicNode) e.endNode).blacklist.remove(e.oppositeEdge);
			((BasicNode) e.startNode).blacklist.remove(e);
		}
	}

}
