package projects.defaultProject.models.distributionModels;

import sinalgo.configuration.Configuration;
import sinalgo.configuration.CorruptConfigurationEntryException;
import sinalgo.models.DistributionModel;
import sinalgo.nodes.Position;
import sinalgo.tools.statistics.Distribution;
import sinalgo.tools.statistics.UniformDistribution;

/**
 * 
 * ConnectedUDG : dumb distribution model that tries random distribution until
 * the corresponding UDG is connected. Should not be any statistical bias for
 * now.
 * 
 * @author rjamet
 * 
 */
public class ConnectedUDG extends DistributionModel {

	int tries;
	double radius;
	int dimX = Configuration.dimX;
	int dimY = Configuration.dimY;

	private Distribution rX = new UniformDistribution(0, dimX);
	private Distribution rY = new UniformDistribution(0, dimY);

	/**
	 * Stores the last valid graph seen, to be feeded back via getPosition
	 */
	Graph lastGraph;
	int lastIndex = 0;
	int lastIndexMax = 0;

	private class Graph { // generate an udg

		public double n[][]; // node
		public boolean e[][]; // edge

		private boolean link(int n1, int n2) {
			// 2D only
			return (radius > Math.sqrt((n[n1][0] - n[n2][0])
					* (n[n1][0] - n[n2][0]) + (n[n1][1] - n[n2][1])
					* (n[n1][1] - n[n2][1])));
		}

		public Graph() {
			n = new double[numberOfNodes][3];// X - Y - seen_connected
			e = new boolean[numberOfNodes][numberOfNodes];
			for (int i = 0; i < numberOfNodes; i++) {
				n[i][0] = rX.nextSample();
				n[i][1] = rY.nextSample();
				n[i][2] = 0; // Not the Z-coord
			}

			for (int j = 0; j < numberOfNodes; j++) {
				for (int i = j; i < numberOfNodes; i++) {
					if (link(i, j)) {
						e[i][j] = true;
						e[j][i] = true;
					} else {
						e[i][j] = false;
						e[j][i] = false;
					}
				}
			}
		}

		public boolean connected() {
			int[] toVisit = new int[numberOfNodes + 1];
			toVisit[0] = 0; // unvisited nodes
			int toVisitLength = 1;
			int i = 0;
			while (i < toVisitLength) {
				int node = toVisit[i++];
				for (int next = 0; next < numberOfNodes; next++) {
					if (e[node][next] && n[next][2] == 0) {
						n[next][2] = 1; // it has been visited
						toVisit[toVisitLength++] = next; // flooding
					}
				}
			}

			// return 0 if a node isn't visited (=> unconnected)
			for (double[] node : n)
				if (node[2] == 0)
					return false;
			return true;
		}
	}

	@Override
	public void initialize() {

		try {
			radius = Configuration.getDoubleParameter("UDG/rMax");
		} catch (CorruptConfigurationEntryException e) {
			// maybe do something
		}

		tries = 100000 / numberOfNodes;
		for (int i = 0; i < tries; i++) {
			Graph g = new Graph();
			if (g.connected()) {
				lastGraph = g;
				lastIndex = 0;
				lastIndexMax = numberOfNodes - 1;
				return;
			}
		}

		throw new RuntimeException("Couldn't generate a connected graph");

	}

	@Override
	public Position getNextPosition() {
		Position p = new Position(
				lastGraph.n[lastIndex][0],
				lastGraph.n[lastIndex][1], 
				0 ); 
		lastIndex++;
		return p;
	}

}
