package projects.defaultProject.nodes.edges;

import java.awt.Color;
import java.awt.Graphics;

import projects.TLCNS.nodes.nodeImplementations.TLCNSNode;
import projects.common.nodes.BasicNode;
import projects.common.nodes.BasicTabuNode;
import sinalgo.gui.helper.Arrow;
import sinalgo.gui.transformation.PositionTransformation;
import sinalgo.nodes.Position;
import sinalgo.nodes.edges.BidirectionalEdge;
import sinalgo.nodes.edges.Edge;

/**
 * Edge to be used with BasicTabuNodes, displays edges faded according to the
 * probability of selection
 * 
 * @author rjamet
 * 
 */
public class FadedTreeDisplayingBidirectionnalEdge extends BidirectionalEdge {

	private double quota() {

		double sum = 0;
		double sumdegrees = 0;
		double me = 0;
		if (this.startNode instanceof BasicTabuNode) {
			if (TLCNSNode.rollingCounters
					&& this.startNode instanceof TLCNSNode) {

				TLCNSNode n = (TLCNSNode) this.startNode;

				for (Edge e : n.outgoingConnections) {
					if (n instanceof BasicTabuNode) {

						sum += Math.pow((n.tl_routing.count(e.endNode)+1.0/n.outgoingConnections.size()),
								BasicTabuNode.alpha);
						if (e.equals(this)) {
							me = Math.pow((n.tl_routing.count(e.endNode)+1.0/n.outgoingConnections.size()),
									BasicTabuNode.alpha);
						}

					} else {
						sum += 1;
						if (e.equals(this)) {
							me += 1;
						}
					}
				}
			} else {
				BasicTabuNode n = (BasicTabuNode) this.startNode;

				if (BasicNode.useLD) {
					for (Edge e : n.outgoingConnections)
						sumdegrees += Math
								.pow(n.counters.get(e)
										/ Math.sqrt(e.endNode.outgoingConnections
												.size()), BasicTabuNode.alpha);
				}

				for (Edge e : n.outgoingConnections) {
					if (n instanceof BasicTabuNode && n.counters != null
							&& n.counters.get(e) != null) {
						if (BasicNode.useLD) {
							sum += Math
									.pow((n.counters.get(e))
											/ Math.sqrt(e.endNode.outgoingConnections
													.size()),
											BasicTabuNode.alpha)
									/ sumdegrees;
							if (e.equals(this)) {
								me = Math
										.pow((n.counters.get(e))
												/ Math.sqrt(e.endNode.outgoingConnections
														.size()),
												BasicTabuNode.alpha)
										/ sumdegrees;
							}
						} else {
							sum += Math.pow((n.counters.get(e)),
									BasicTabuNode.alpha);
							if (e.equals(this)) {
								me = Math.pow((n.counters.get(e)),
										BasicTabuNode.alpha);
							}
						}
					} else {
						sum += 1;
						if (e.equals(this)) {
							me += 1;
						}
					}
				}
			}

			return me / sum;

		} else {
			return -1;
		}

	}

	@Override
	public String toString() {
		return startNode.ID + "->" + endNode.ID + ":" + quota();// ((float)(me /
																// sum));
	}
	@Override
	public Color getColor() {
		double quota = quota();
		if (this.startNode instanceof BasicTabuNode
				&& ((BasicTabuNode) this.startNode).isSink()) {
			return new Color(0, 0, 0, 0);
		}
		if (quota >= 0) {
			return new Color(0, (float) 0.5, 0, (float) ((quota *5 + 5 )/ 10.0));
		} else {
			return Color.WHITE;
		}
	}

	@Override
	public void draw(Graphics g, PositionTransformation pt) {
		Position p1 = startNode.getPosition();
		pt.translateToGUIPosition(p1);
		int fromX = pt.guiX, fromY = pt.guiY; // temporarily store
		Position p2 = endNode.getPosition();
		pt.translateToGUIPosition(p2);

		g.setColor(Color.GRAY);
		g.drawLine(fromX, fromY, pt.guiX + ((fromX - pt.guiX) / 2),
				pt.guiY + ((fromY - pt.guiY) / 2));


		Arrow.drawArrowHead(fromX, fromY, pt.guiX + ((fromX - pt.guiX) / 2),
				pt.guiY + ((fromY - pt.guiY) / 2), g, pt, getColor(),quota());

	}

}
