package Runner::IO;

use strict;
use warnings;
use Carp;
use List::Util qq/sum/;

#use File::Map qw(map_file);

my $tbegin;

BEGIN{ 
	$tbegin = time();
}


sub runner_usage {
  print "Third iteration of the sinalgo simulation runner\n";
  print "Runs some sinalgo simulations, JSON magic, basic computations,". 
        " gnuplot scripts, and a lot of small other things.\n\n";

  print "Usage : ./runner.pl \n".   
        "   -p Project_to_run \n".
        "   -n Name_of_the_run \n".
				"		-N Node creation command (\"-gen 10 TLCN:TLCNNode Random\")\n". 
        "  [-a \"some/Sinalgo/XML=attributes With/some=quotes\"]\n".
        "  [-r Number_of_simulation_runs]\n".
#       "  [-d \"Spaced list of destination machines\" (defaults to localhost)]\n". NEVER IMPLEMENTED
				" You can also use \$RUN in the params, which will be replaced by the good value.\n";

}

sub rprint {
	print "\033[33m";
	tprint(@_);
	print "\033[0m";
}

sub tprint {
	my $tstring = time() - $tbegin;
	$tstring = ' 'x(10-length($tstring)) . $tstring;
	print "[".$tstring."] ".join (" ",@_);
}


#avoid cyclic dependancies
use Runner::Stats;

sub dump_averages($$) {
	my $data = shift;
	my $name = shift;
  



	open(my $average_on_tb, "> ../results/$name/av_tb.data") 
		or croak("Can't open the file ! $!");
	foreach (sort {$a->{t} <=> $b->{t}} @{$data->{average_rolling}}) {
		print $average_on_tb ($_->{t}." ".$_->{h}."\n");
	}
  close($average_on_tb);	


	open(my $average_on_tb_ack, "> ../results/$name/av_tb_ack.data") 
		or croak("Can't open the file ! $!");
	foreach (sort {$a->{t} <=> $b->{t}} @{$data->{average_rolling_ack}}) {
		print $average_on_tb_ack ($_->{t}." ".$_->{h}."\n");
	}
  close($average_on_tb_ack);	


	open(my $tph, "> ../results/$name/tph.data") 
		or croak("Can't open the file ! $!");
	foreach (sort {$a->{t} <=> $b->{t}} @{$data->{average_rolling}}) {
		print $tph ($_->{t}." ".$_->{tph}."\n");
	}
  close($tph);	

#	open(my $en, "> ../results/$name/energy.data") 
#		or croak("Can't open the file ! $!");
#	foreach (sort {$a->{tb} <=> $b->{tb}} @{$data->{energy_rolling}}) {
#		print $en ($_->{tb}." ".$_->{te}." ".$_->{nbh}." ".$_->{cnbh}." ".$_->{pnbh}." ".$_->{cpnbh}."\n");
#	}
#	close($en);	

	open(my $delivery_on_tb, "> ../results/$name/de_tb.data") 
		or croak("Can't open the file ! $!");	
	foreach (sort {$a->{t} <=> $b->{t}} @{$data->{delivery_rolling}}) {
		print $delivery_on_tb ($_->{t}." ".$_->{ok}." ".($_->{eaten}+$_->{ok})." ".(($_->{eaten}) + ($_->{ok}) + ($_->{ttl}))."\n");
	}
	close($delivery_on_tb);



	open(my $delivery_on_tb_ack, "> ../results/$name/de_tb_ack.data") 
		or croak("Can't open the file ! $!");	
	foreach (sort {$a->{t} <=> $b->{t}} @{$data->{delivery_rolling_ack}}) {
		print $delivery_on_tb_ack (
			$_->{t}." ".
			$_->{ok}." ".
			($_->{ok} + $_->{for})." ".
			($_->{eaten}+$_->{ok}+$_->{for})." ".
			(($_->{eaten}) + ($_->{ok}) + ($_->{ttl}) + $_->{for})."\n");
	}
	close($delivery_on_tb_ack);


	return 1;
}

sub dump_raw($$) {
	my $data = shift;
	my $name = shift;
  open(my $raw_points, "> ../results/$name/raw.data");
  print $raw_points $_->{tb}." ".$_->{h}."\n" foreach (grep {$_->{type} eq "basic"} @{$data->{msg_data}});
  close($raw_points);
	return 1;
}

sub dump_json {
	my $data = shift;
	my $name = shift;
	my $tmp = $data->{msg_data}; 
	$data->{msg_data}=[];
	if (open (my $fout,"> ../results/$name/dump.json")) {
	  print $fout JSON::XS->new->pretty->shrink->encode($data);
	  close($fout);
		$data->{msg_data}=$tmp;
		return 1;
	} else {
		$data->{msg_data}=$tmp;

	  carp "Couldn't open \"../results/$name/dump.json\" : $!";
		return 0;
	}
}

sub dump_globals {
	my $data = shift;
	my $name = shift;

  open(my $glob, "> ../results/$name/glob.json");
		print $glob JSON::XS->new->pretty->encode($data->{simulation});
	close $glob;
	return 0;

}

sub dump_node_data{
	my $data = shift;
	my $name = shift;

  open(my $glob, "> ../results/$name/node_data.json");
		print $glob JSON::XS->new->pretty->encode($data->{nodes});
	close $glob;

	open ($glob, "> ../results/$name/classes.data");
		print $glob $name." ".join (" ",	map { $_ / (sum @{$data->{simulation}->{classes}}) } @{$data->{simulation}->{classes}});
	close $glob;

	open ($glob, "> ../results/$name/moustache.data");
		print $glob $name." ".join (" ",@{$data->{simulation}->{boxplot}});
	close $glob;
	return 0;


}

#sub import_json { #Untested yet
#	my $filename = shift; 
#	map_file my $data, $filename;
#	advise $data, "sequential";
#	return JSON::XS->new->decode($data);
#}

return 1;
