package Runner::Stats;


use strict;
use warnings;

use Runner::IO;
use List::Util qw/reduce sum max min/;


my $RA_SAMPLES = 500;
my $RD_SAMPLES = 500;
my $RE_TIMEWINDOW = 100;

#Used to compute all the means and stddev and so on
sub computeOnMessages($$) {

	my $data = shift;
	my $command = shift;

	if (!exists $data->{inc_vars}) {
		#first run, create the data
		$data->{inc_vars}->{last_id_RA} = 0;
		$data->{inc_vars}->{last_id_RA_ack} = 0;
		$data->{inc_vars}->{last_id_RD} = 0;
		$data->{inc_vars}->{last_id_RD_ack} = 0;
	} else {
		#reuse old data
	}
	
	#Runner::IO::tprint "### Extracting the uniques send times\n";

	$data->{average_rolling} = [] unless exists $data->{average_rolling};
	$data->{average_rolling_ack} = [] unless exists $data->{average_rolling_ack};
	$data->{delivery_rolling} = [] unless exists $data->{delivery_rolling};
	$data->{delivery_rolling_ack} = [] unless exists  $data->{delivery_rolling_ack};
	$data->{energy_rolling} = [] unless exists $data->{energy_rolling};

	###################################
	# NUMBER OF HOPS FOR MSG
	###################################

	#Runner::IO::tprint "### Computing averages on send times\n";
	my @sorted = grep { 
								$_->{d} == 1 && 
								$_->{id} > $data->{inc_vars}->{last_id_RA} &&
								$_->{type} eq "basic"
							} @{$data->{msg_data}};

	while( scalar @sorted >= $RA_SAMPLES) {
		my @cur = splice @sorted, 0, $RA_SAMPLES;
		my $h_r =  ( sum map{$_->{h}} @cur ) / @cur;
		my $t_r =  ( sum map{$_->{tb}} @cur ) / @cur;
		my $tph_r =( sum map{ ($_->{te} - $_->{tb})/$_->{h} } @cur )  / @cur;

		push @{$data->{average_rolling}}, {"h" => $h_r, "t" => $t_r, "tph" => $tph_r};
		$data->{inc_vars}->{last_id_RA} = max map{$_->{id}} @cur;
	}	
	

	###################################
	# DELIVERY RATE FOR MSG
	###################################

	#Runner::IO::tprint "### Computing average delivery rates\n";
	@sorted = grep { 
								$_->{type} eq "basic" &&
								$_->{id} > $data->{inc_vars}->{last_id_RD}
							} @{$data->{msg_data}};
	while (scalar @sorted >= $RD_SAMPLES) {
		my @cur = splice @sorted, 0, $RD_SAMPLES;
		my $eaten_r = ( scalar grep { $_->{d} == 0 } @cur ) / @cur;
		my $ok_r =    ( scalar grep { $_->{d} == 1 } @cur ) / @cur;
		my $ttl_r =   ( scalar grep { $_->{d} == 2 } @cur ) / @cur;
		my $other_r = ( scalar grep { $_->{d} >= 3 } @cur ) / @cur;

		my $t_r =  ( sum  map{$_->{tb}} @cur ) / @cur;
		push @{$data->{delivery_rolling}}, 
			{"eaten" => $eaten_r, "ok" => $ok_r,"ttl"=>$ttl_r, "other"=>$other_r, "t" => $t_r};		
				
		$data->{inc_vars}->{last_id_RD} = max map{$_->{id}} @cur;
	}

	####################################
	# NBHOPS + DELIVERY RATE FOR ACKS
	####################################

	if (grep {$_->{type} eq "ack"} @{$data->{msg_data}}) {
		
		my @sorted = grep {
   	          ($_->{d} == 1 || $_->{d} == 3) &&
   	          $_->{id} > $data->{inc_vars}->{last_id_RA_ack} &&
   	          $_->{type} eq "ack"
   	        } @{$data->{msg_data}};

 		while( scalar @sorted >= $RA_SAMPLES) {
	 		my @cur = splice @sorted, 0, $RA_SAMPLES;
 	  	my $h_r =  ( sum map{$_->{h}} @cur ) / @cur;
 	  	my $t_r =  ( sum map{$_->{tb}} @cur ) / @cur;
 	  	my $tph_r =( sum map{ ($_->{te} - $_->{tb})/$_->{h} } @cur )  / @cur;

   		push @{$data->{average_rolling_ack}}, {"h" => $h_r, "t" => $t_r, "tph" => $tph_r};
   		$data->{inc_vars}->{last_id_RA_ack} = max map{$_->{id}} @cur;

		}

		#Runner::IO::tprint "### Computing average delivery rates for acks\n";
		@sorted = grep { 
									$_->{id} > $data->{inc_vars}->{last_id_RD_ack} &&
									$_->{type} eq "ack" 
								} @{$data->{msg_data}};
		while (scalar @sorted >= $RD_SAMPLES) {
			my @cur = splice @sorted, 0, $RD_SAMPLES;
			my $eaten_r = ( scalar grep { $_->{d} == 0 } @cur ) / @cur;
			my $ok_r =    ( scalar grep { $_->{d} == 1 } @cur ) / @cur;
			my $ttl_r =   ( scalar grep { $_->{d} == 2 } @cur ) / @cur;
			my $forgot_r =( scalar grep { $_->{d} == 3 } @cur ) / @cur;
			my $other_r = ( scalar grep { $_->{d} >= 4 } @cur ) / @cur;

			my $t_r =  ( sum  map{$_->{tb}} @cur ) / @cur;
			push @{$data->{delivery_rolling_ack}}, {
				"eaten" => $eaten_r, 
				"ok" => $ok_r,
				"ttl"=>$ttl_r, 
				"for"=>$forgot_r,
				"other"=>$other_r,
				"t" => $t_r
			};

			$data->{inc_vars}->{last_id_RD_ack} = max map{$_->{id}} @cur;
		}

	}

	###################################
	# TOPSECRET ENERGY STUFF
	# Will probably break when cutting in parts
	###################################
	
	#Runner::IO::tprint "### Retrocombulating the bootstraps\n";
#	@sorted = grep {
#							$_->{id} > $data->{inc_vars}->{last_id_RE}
#						} @{$data->{msg_data}};
#
#	my $twmin = 0; my $twmax = $RE_TIMEWINDOW;
#	my $cnbh = 0;
#	my $cpnbh = 0;
#	while (@sorted > 0) {
#
#		my @cur;
#		while (@sorted > 0 && $sorted[0]->{tb} < $twmax) {
#			push @cur, (shift @sorted);
#		}
#
#		if (@cur) {
#			my $nbh = ( sum map { $_->{h} } @cur );
#			my $pnbh = $nbh * (scalar grep {$_->{type} eq "basic"} @cur) / (0.001 + scalar grep {$_->{type} eq "basic" && $_->{d}==1} @cur ); 
#			$cnbh += $nbh;
#			$cpnbh += $pnbh;
#			push @{$data->{energy_rolling}}, {"nbh" => $nbh, "cnbh" => $cnbh, "pnbh" => $pnbh, "tb" => $twmin, "te" => $twmax, "cpnbh" => $cpnbh};
#		} else {
#			push @{$data->{energy_rolling}}, {"nbh" => 0,    "cnbh" => $cnbh, "pnbh" => 0,     "tb" => $twmin, "te" => $twmax, "cpnbh" => $cpnbh};
#		}
#		$twmin += $RE_TIMEWINDOW;
#		$twmax += $RE_TIMEWINDOW;
#		#WILL BREAK FIXME 
#		$data->{inc_vars}->{last_id_RE} = max map{$_->{id}} @cur;
#	}



	my $last_useful_id = min(
		(	
			$data->{inc_vars}->{last_id_RA},
			$data->{inc_vars}->{last_id_RA_ack},
			$data->{inc_vars}->{last_id_RD},
			$data->{inc_vars}->{last_id_RD_ack}
		)
	);

	my @newdata = grep {$_->{id} > $last_useful_id} @{$data->{msg_data}};
	undef @{$data->{msg_data}};
	$data->{msg_data} = \@newdata;

	1;
}











sub computeAtTheEnd($$) {

	my $data = shift;
	my $command = shift;

	####################################
	# STUFF COMPUTED AFTER EVERYTHING ELSE
	####################################

	if (@{$data->{average_rolling}}) {
		$data->{simulation}->{mean_nbh} =
			( sum map {$_->{h}} @{$data->{average_rolling}} ) / @{$data->{average_rolling}};
	} else {
		Runner::IO::tprint "### Problem : too few delivered messages to make an average of the nbhops\n";
		$data->{simulation}->{mean_nbh} = -1;	  
	}

	if (@{$data->{delivery_rolling_ack}}) {
		$data->{simulation}->{total_dr_ack_received} =
			( sum map {$_->{ok} + $_->{for}} @{$data->{delivery_rolling_ack}} ) / @{$data->{delivery_rolling_ack}};
		$data->{simulation}->{total_dr_ack_remembered} =
			( sum map {$_->{ok}}             @{$data->{delivery_rolling_ack}} ) / @{$data->{delivery_rolling_ack}};
	}

	#Fairness as we do it
	my @cleandata = 
		grep {$_->{t} ne "blackhole" && $_->{t} ne "sink"} @{$data->{nodes}};  
	my $sqfairness = ((sum map {$_->{dr}} @cleandata) / @cleandata)**2;
	my $fairnesssq = (sum map {$_->{dr}**2} @cleandata) / @cleandata;
	$data->{simulation}->{fairness} = sqrt($fairnesssq - $sqfairness);

	#Moustache plot DR/node
	@cleandata = 
		sort {$a <=> $b}
		map {$_->{dr}}
		grep {$_->{t} ne "blackhole" && $_->{t} ne "sink"} @{$data->{nodes}};
	$data->{simulation}->{boxplot}->[0] = $cleandata[int (@cleandata*0.1)];
	$data->{simulation}->{boxplot}->[1] = $cleandata[int (@cleandata*0.25)];
	$data->{simulation}->{boxplot}->[2] = $cleandata[int (@cleandata*0.5)];
	$data->{simulation}->{boxplot}->[3] = $cleandata[int (@cleandata*0.75)];
	$data->{simulation}->{boxplot}->[4] = $cleandata[int (@cleandata*0.9)];

	#Stacks ORANGE-like
	@cleandata = grep {$_->{t} !~ /blackhole/ && $_->{t} !~ /sink/} @{$data->{nodes}};  
	$data->{simulation}->{classes}->[0] = grep {$_->{dr} <= 0} @cleandata;
	$data->{simulation}->{classes}->[1] = grep {$_->{dr} > 0.0    && $_->{dr} <= 0.33} @cleandata;
	$data->{simulation}->{classes}->[2] = grep {$_->{dr} > 0.33 && $_->{dr} <= 0.66} @cleandata;
	$data->{simulation}->{classes}->[3] = grep {$_->{dr} > 0.66 && $_->{dr} < 1} @cleandata;
	$data->{simulation}->{classes}->[4] = grep {$_->{dr} >= 1} @cleandata;
	$data->{simulation}->{total_dr} =
		( sum map {$_->{ok}} @{$data->{delivery_rolling}} ) / @{$data->{delivery_rolling}};


	####################################
	# MISC NETWORK AND SETTINGS
	####################################

	$data->{simulation}->{av_sink_degree} = 
		(sum map {$_->{d}} grep {$_->{t} eq "sink"} @{$data->{nodes}}) / (grep {$_->{t} eq "sink"} @{$data->{nodes}}) ;

	$data->{simulation}->{average_degree} = 
		(sum map {$_->{d}} @{$data->{nodes}}) / @{$data->{nodes}} ;

	$data->{simulation}->{stddev_degree} = 
		sqrt(((sum map {$_->{d}**2} @{$data->{nodes}}) / @{$data->{nodes}} ) - $data->{simulation}->{average_degree}**2);

	$data->{simulation}->{command} = $command;
	

	if ($command =~ m/Messages\/Number=(\d+)/) {
		$data->{simulation}->{number_messages} = $1;
	}	
	
	return 1;
}



1;
