# Regroup and stat results of related simulation runs,
# given as parameter. Generates the results in its
# own repertory.

use Runner::Stats;
use List::Util qw/reduce sum max min/;
use JSON::XS;

my $name = shift;

my @datadirs;
my @group_nodes             = ();
my $group_dr                = 0;
my $group_fair              = 0;
my $group_nbh               = 0;
my $group_dr_ack_received   = 0;
my $group_dr_ack_remembered = 0;
my $group_o		              = 0;

sub work {
	my $dir = $_;

	$dir =~ s/\/$//;

	my $name = $dir;
	if ( $dir =~ /\/([\d\w_\-\.]+)\/?$/ ) {
		$name = $1;
	}

	open( my $SIMFILE, "$dir/glob.json" ) or die("$dir not found");
	my $json_sim_data = join "", <$SIMFILE>;
	close $SIMFILE;
	my $sim_data = JSON::XS->new->relaxed->decode($json_sim_data);

	open( my $NODEFILE, "$dir/node_data.json" );
	my $json_node_data = join "", <$NODEFILE>;
	close $NODEFILE;
	my $node_data = JSON::XS->new->relaxed->decode($json_node_data);

	@group_nodes = ( @group_nodes, @{$node_data} );
	$group_dr                += $sim_data->{total_dr};
	$group_fair              += $sim_data->{fairness};
	$group_dr_ack_received   += $sim_data->{total_dr_ack_received};
	$group_dr_ack_remembered += $sim_data->{total_dr_ack_remembered};
	$group_nbh               += $sim_data->{mean_nbh};
	$group_o	             	 += $sim_data->{mean_overhead};

	return 0;
}

sub finish {

	my @cleandata =
	  sort { $a <=> $b }
	  map  { $_->{dr} }
	  grep { $_->{t} ne "SFnode" && $_->{t} ne "blackhole" && $_->{t} ne "sink" } @group_nodes;
	$boxplot->[0] = $cleandata[ int( @cleandata * 0.1 ) ];
	$boxplot->[1] = $cleandata[ int( @cleandata * 0.25 ) ];
	$boxplot->[2] = $cleandata[ int( @cleandata * 0.5 ) ];
	$boxplot->[3] = $cleandata[ int( @cleandata * 0.75 ) ];
	$boxplot->[4] = $cleandata[ int( @cleandata * 0.9 ) ];

	#Stacks of 4
	@cleandata =
	  grep { $_->{t} ne "SFnode" && $_->{t} !~ /blackhole/ && $_->{t} !~ /sink/ } @group_nodes;
	$classes->[0] = grep { $_->{dr} <= 0 } @cleandata;
	$classes->[1] = grep { $_->{dr} > 0.0 && $_->{dr} <= 0.33 } @cleandata;
	$classes->[2] = grep { $_->{dr} > 0.33 && $_->{dr} <= 0.66 } @cleandata;
	$classes->[3] = grep { $_->{dr} > 0.66 && $_->{dr} < 1 } @cleandata;
	$classes->[4] = grep { $_->{dr} >= 1 } @cleandata;

	#Stacks of 12
	@cleandata =
	  grep { $_->{t} ne "SFnode" && $_->{t} !~ /blackhole/ && $_->{t} !~ /sink/ } @group_nodes;
	$classes10->[0] = grep { $_->{dr} <= 0 } @cleandata;
	for $i ( 1 .. 9 ) {
		$classes10->[$i] =
		  grep { $_->{dr} > ( ( $i - 1 ) / 10 ) && $_->{dr} <= ( $i / 10 ) }
		  @cleandata;
	}
	$classes10->[10] = grep { $_->{dr} > 0.9 && $_->{dr} < 1 } @cleandata;
	$classes10->[11] = grep { $_->{dr} >= 1 } @cleandata;

	my $glob;

	open( $glob, ">> ./glob.data" );
	print $glob ( $name . " "
		  . ( $group_dr / @datadirs ) . " "
		  . ( $group_nbh / @datadirs ) . " "
		  . ( $group_dr_ack_received / @datadirs ) . " "
		  . ( $group_dr_ack_remembered / @datadirs ) . " "
		  . ( $group_o / @datadirs )
		  . "\n" );
	print "$name \& ". sprintf('$%3.3f$ & $%3.3f$', $group_dr / @datadirs , $group_fair / @datadirs )."\n";
	close $glob;

	open( $glob, ">> ./classes.data" );
	print $glob $name . " "
	  . join( " ", map { $_ / ( sum @{$classes} ) } @{$classes} ) . "\n";
	close $glob;

	open( $glob, ">> ./classes10.data" );
	print $glob $name . " "
	  . join( " ", map { $_ / ( sum @{$classes10} ) } @{$classes10} ) . "\n";
	close $glob;

	open( $glob, ">> ./moustache.data" );
	print $glob $name . " " . join( " ", @{$boxplot} ) . "\n";
	close $glob;

}

sub draw {
return;
	my $plot = <<END2;
#!/usr/bin/gnuplot
set output "DR_moustache.svg"
set title "Delivery rates"
set term svg enhanced size 1000 400
set boxwidth 0.5 relative
set datafile missing '-'
set yrange [0:1.1]
set xtics nomirror rotate by -45 scale 0
unset key
unset grid
plot "moustache.data" u :3:2:6:5:xticlabel(1) w candlesticks whiskerbars, '' u :4:4:4:4 w candlesticks;
END2

	open( $GNUPLOT, "| gnuplot" );
	print $GNUPLOT $plot . "\n";
	close($GNUPLOT);

	$plot = <<END2;
#!/usr/bin/gnuplot
set output "DR_stack.svg"
set title "Delivery rate classes"
set term svg enhanced size 1000 400
set style data histograms 
set style fill solid 1.0 border -1
set style histogram rowstacked
set boxwidth 0.8 relative
set datafile missing '-'
set yrange [0:1.1]
unset grid
set xtics nomirror rotate by -45 scale 0
plot
END2

	chomp $plot;

	$i = 1;
	$plot .=
	    "\"classes.data\" u 2:xticlabel(1) lc rgb(\"#000000\") t \"A\", "
	  . "''  u 3 t \"B\" lc rgb(\"#FF0000\"),"
	  . "''  u 4 t \"C\" lc rgb(\"#AA5500\"),"
	  . "''  u 5 t \"D\" lc rgb(\"#55AA00\"),"
	  . "''  u 6 t \"E\" lc rgb(\"#00FF00\");";

	open( $GNUPLOT, "| gnuplot" );
	print $GNUPLOT $plot . "\n";
	close($GNUPLOT);

	$plot = <<END3;
#!/usr/bin/gnuplot
set output "DR_stack10.svg"
set title "Delivery rate classes"
set term svg enhanced size 1000 400
set style data histograms 
set style fill solid 1.0 border -1
set style histogram rowstacked
set boxwidth 0.4 relative
set datafile missing '-'
set yrange [0:1.1]
unset grid
set xtics nomirror rotate by -45 scale 0
plot
END3

	chomp $plot;

	$i = 1;
	$plot .=
	    "\"classes10.data\" u 2:xticlabel(1) lc rgb(\"#0F0000\") t \"0\", "
	  . "''  u 3 t \"0-.1\" lc rgb(\"#2F0000\"),"
	  . "''  u 4 t \".1-.2\" lc rgb(\"#4F0000\"),"
	  . "''  u 5 t \".2-.3\" lc rgb(\"#6F0000\"),"
	  . "''  u 6 t \".3-.4\" lc rgb(\"#7F0000\"),"
	  . "''  u 7 t \".4-.5\" lc rgb(\"#6F2200\"),"
	  . "''  u 8 t \".5-.6\" lc rgb(\"#4F4400\"),"
	  . "''  u 9 t \".6-.7\" lc rgb(\"#2F6600\"),"
	  . "''  u 10 t \".7-.8\" lc rgb(\"#0F8800\"),"
	  . "''  u 11 t \".8-.9\" lc rgb(\"#00AA00\"),"
	  . "''  u 12 t \".9-1.0\" lc rgb(\"#00CC00\"),"
	  . "''  u 13 t \"1\" lc rgb(\"#00EE00\"),"
	  . "'glob.data' u :2 with points lt 5 pt 7 lc rgb(\"white\");";

	open( $GNUPLOT, "| gnuplot" );
	print $GNUPLOT $plot . "\n";
	close($GNUPLOT);

}

my $first = 1;
while ( $a = shift ) {
	if ($first) { $first = 0; print "$a\n";}
	push @datadirs, $a;
}

for (@datadirs) {
	work $_;
}
if (@datadirs) {
	finish;
}
draw;
print "\n";
