#!/usr/bin/perl
use List::Util qw/max/;
use JSON::XS;
srand;

my $dest = "./multisim_plots";
`mkdir -p $dest`;

my $term = "set term svg size 1000 400";



#parse input
my @datadirs;
my $d;
while($d = shift){
	push(@datadirs,$d);
}

#@datadirs = sort {$a cmp $b} @datadirs;

my $name = "glob.json";

my %tab;
for (@datadirs) {
	open (my $data, "< $_/$name") or die("Can't find $_/$name : $!");
	$tab{$_} = JSON::XS->new->relaxed->decode(join("",<$data>));
	close ($data);
}
open(my $t, "> $dest/table.html") or die ("can't open $dest/table.html : $!");
my @a = keys %tab;
my $e = $a[0];
print $t "<table border=\"1\">";
print $t "<tr><td /><th>".join("</th><th>",map { $_ } grep {$_ !~ /command/} sort keys %{$tab{$e}})."</th></tr>\n";

@a = sort {substr($a,23,1024) <=> substr($b,23,1024)}  keys %tab;
for $e (@a) {
#for $e (sort {$a cmp $b} keys %tab) {
	print $t "<tr><th>$e</th><td>".join("</td><td>",map { $tab{$e}->{$_} } grep {$_ !~ /command/} sort keys %{$tab{$e}})."</td></tr>\n";
}
print $t "</table>";
close $t;


#plots
$name = "de_tb.data";
my $size = 5.0 / max(5,(scalar @datadirs / 5)); 
if (@datadirs > 30) {
	$key = "unset key";
} else {
	$key = "set key bottom right";
}

my $plot = <<END; 
#!/usr/bin/gnuplot
set output "$dest/delivery.svg"
set title "DR of different algorithms on a 200 nodes, degree 8 network against 5 pct BH and 5 pct WH"
$term
set pointsize 0.5
set samples 1000
$key
set xrange [0:]
set yrange [0:1]
set xlabel "Time"
set ylabel "Delivery rate"
unset grid
plot 
END

chomp $plot; my $i = 1;
for (@datadirs) {
	$_ =~ /L([\d\.]+)_/;
#	$plot .= "\"$_/$name\" u 1:2 smooth sbezier with lines lc $i lw $size t \"$1\", ";
	$plot .= "\"$_/$name\" u 1:2 with lines lc $i lw 1 t \"$1\", ";
	$plot .= "\"$_/$name\" u 1:2 with points ps 0.1 pt 7 lc $i notitle, ";
	$i++;
}
#$plot .= "\"$_/$name\" u 1:2 with lines lw $size t \"$_\", " for @datadirs;
$plot =~ s/,\s+$//;

open($GNUPLOT,"| gnuplot");
print $plot;
print $GNUPLOT $plot;
close($GNUPLOT);



$name = "de_tb.data";
my $size = 5.0 / max(5,(scalar @datadirs / 5)); 
if (@datadirs > 30) {
	$key = "unset key";
} else {
	$key = "set key bottom right";
}

my $plot = <<END; 
#!/usr/bin/gnuplot
set output "$dest/delivery.eps"
set title "DR of different algorithms on a 200 nodes, degree 8 network against 5 pct BH and 5 pct WH"
set term postscript eps monochrome enhanced size 7,3 18;
$key
set xrange [0:]
set yrange [0:1]
set xlabel "The x^th message has been processed (either delivered or lost)"
set ylabel "Avg. delivery rate on the window (x-10^4..x)"
unset grid
plot 
END

chomp $plot; my $i = 1;
for (@datadirs) {
	$_ =~ /SWITCH-(\d+)-/;
#	$plot .= "\"$_/$name\" u 1:2 smooth sbezier with lines lc $i lw $size t \"$1\", ";
	$plot .= "\"$_/$name\" u 1:2 with lines lt $i t \"Switch every $1 msg\", ";
	$i++;
}
#$plot .= "\"$_/$name\" u 1:2 with lines lw $size t \"$_\", " for @datadirs;
$plot =~ s/,\s+$//;

open($GNUPLOT,"| gnuplot");
print $plot;
print $GNUPLOT $plot;
close($GNUPLOT);









#################
$plot = <<END2; 
#!/usr/bin/gnuplot
set output "$dest/nbhops.svg"
set title "Average #hops"
$term
set pointsize 0.5
set samples 1000
$key
set xrange [0:500000]
#set xrange [0:30000]
set yrange [0:40]
unset grid
plot 
END2

chomp $plot;

$name = "av_tb.data";
$i = 1;
for (@datadirs) {


	$_ =~ /STB\-([\w\d]+?)_/;
	$plot .= "\"$_/$name\" u 1:2 smooth sbezier with lines lc $i lw $size t \"$1\", ";
	$plot .= "\"$_/$name\" u 1:2 with lines lc $i lw 0.1 notitle, ";
	$i++;
}
$plot =~ s/,\s+$//;

open($GNUPLOT,"| gnuplot");
print $GNUPLOT $plot."\n";
close($GNUPLOT);

#####################
$plot = <<END2; 
#!/usr/bin/gnuplot
set output "$dest/nbhops_MSG.svg"
set title "Average #hops per msg"
$term
set pointsize 0.5
set samples 1000
$key
set xrange [0:]
#set xrange [0:30000]
set yrange [0:40]
unset grid
plot 
END2

chomp $plot;

$name = "av_tb.data";
$i = 1;
for (@datadirs) {

	$_ =~ /STB\-([\w\d]+?)_/;
	$plot .= "\"$_/$name\" u :2 smooth sbezier with lines lc $i lw $size t \"$1\", ";
	$plot .= "\"$_/$name\" u :2 with lines lc $i lw 0.1 notitle, ";
	$i++;
}
$plot =~ s/,\s+$//;

open($GNUPLOT,"| gnuplot");
print $GNUPLOT $plot."\n";
close($GNUPLOT);

######################"
$plot = <<END2; 
#!/usr/bin/gnuplot
set output "$dest/tph.svg"
set title "Average time/hop"
$term
set pointsize 0.5
set samples 1000
$key
#set xrange [0:30000]
set yrange [0:]
unset grid
plot 
END2

chomp $plot;

$name = "tph.data";
$i = 1;
for (@datadirs) {
#	$plot .= "\"$_/$name\" u 1:2 smooth sbezier with lines lc $i lw $size t \"$_\", ";
	$plot .= "\"$_/$name\" u 1:2 with lines lc $i lw $size t \"$_\", ";
	$plot .= "\"$_/$name\" u 1:2 with points ps 0.1 pt 7 lc $i notitle, ";
	$i++;
}
$plot =~ s/,\s+$//;

open($GNUPLOT,"| gnuplot");
print $GNUPLOT $plot."\n";
close($GNUPLOT);

######################"
#$plot = <<END2; 
#!/usr/bin/gnuplot
#set output "nbhops_ack.svg"
#set title "Nbhop ack"
#$term
#set pointsize 0.5
#set samples 1000
#$key
#set xrange [0:30000]
#set yrange [0:]
#unset grid
#plot 
#END2

#chomp $plot;

#$name = "av_tb_ack.data";
#$i = 1;
#for (@datadirs) {
#	$plot .= "\"$_/$name\" u 1:2 smooth sbezier with lines lc $i lw $size t \"$_\", ";
#	$plot .= "\"$_/$name\" u 1:2  with lines lc $i lw $size t \"$_\", ";
#	$plot .= "\"$_/$name\" u 1:2 with points ps 0.1 pt 7 lc $i notitle, ";#
#	$i++;
#}
#$plot =~ s/,\s+$//;

#open($GNUPLOT,"| gnuplot");
#print $GNUPLOT $plot."\n";
#close($GNUPLOT);

##################"
$name = "classes.data";
`echo '' > $name`;
for (@datadirs) {
	`cat $_/$name >> $name`;
	`echo '' >> $name`;
}

$plot = <<END2;
#!/usr/bin/gnuplot
set output "$dest/DR_stack.svg"
set title "Delivery rate classes"
$term
set style data histograms 
set style fill solid 1.0 border -1
set style histogram rowstacked
set boxwidth 0.8 relative
set datafile missing '-'
set yrange [0:1.1]
$key
unset grid
set xtics nomirror rotate by -45 scale 0
plot
END2

chomp $plot;

$i=1;
$plot .= "\"$name\" u 2:xticlabel(1) lc rgb(\"#000000\") t \"A\", ".
					 "''  u 3 t \"B\" lc rgb(\"#FF0000\"),".
					 "''  u 4 t \"C\" lc rgb(\"#AA5500\"),".
					 "''  u 5 t \"D\" lc rgb(\"#55AA00\"), ".
					 "''  u 6 t \"E\" lc rgb(\"#00FF00\");";

open($GNUPLOT,"| gnuplot");
print $GNUPLOT $plot."\n";
close($GNUPLOT);


$name = "moustache.data";
`echo '' > $name`;
for (@datadirs) {
	`cat $_/$name >> $name`;
	`echo '' >> $name`;
}

$plot = <<END2;
#!/usr/bin/gnuplot
set output "$dest/DR_moustache.svg"
set title "Delivery rates"
$term
set boxwidth 0.5 relative
set datafile missing '-'
set yrange [0:1.1]
set xtics nomirror rotate by -45 scale 0
unset key
unset grid
plot "moustache.data" u :3:2:6:5:xticlabel(1) w candlesticks whiskerbars, '' u :4:4:4:4 w candlesticks;
END2

open($GNUPLOT,"| gnuplot");
print $GNUPLOT $plot."\n";
close($GNUPLOT);

`rm -r *.data`;
